@@ -1,4 +1,4 @@
-/* $Id: axconfig.c,v 1.19 2004/07/13 18:05:45 nachbaur Exp $ */
+/* $Id: axconfig.c,v 1.1 2002/01/13 20:45:08 matts Exp $ */
 
 #ifndef WIN32
 #include <modules/perl/mod_perl.h>
@@ -83,10 +83,9 @@ ax_cleanup_hv(void * hv_v)
 axkit_dir_config *
 new_axkit_dir_config (pool *p)
 {
-    axkit_dir_config *new =
+    axkit_dir_config *new = 
         (axkit_dir_config *) ap_palloc(p, sizeof(axkit_dir_config));
-
-    new->debug_tidy = -1;
+    
     new->translate_output = -1;
     new->gzip_output = -1;
     new->log_declines = -1;
@@ -94,33 +93,30 @@ new_axkit_dir_config (pool *p)
     new->no_cache = -1;
     new->debug_level = -1;
     new->dependency_checks = -1;
-    new->ignore_style_pi = -1;
-    new->handle_dirs = -1;
     new->reset_processors = 0;
     new->reset_output_transformers = 0;
     new->reset_plugins = 0;
-
+    
     new->cache_dir = 0;
     new->config_reader_module = 0;
-    new->contentprovider_module = 0;
+    new->provider_module = 0;
     new->styleprovider_module = 0;
-    new->dir_handler_module = 0;
     new->default_style = 0;
     new->default_media = 0;
     new->cache_module = 0;
     new->output_charset = 0;
-    new->trace_intermediate = 0;
-
+    
     /* complex types */
     new->type_map = NULL;
     new->processors = NULL;
+    new->dynamic_processors = NULL;
     new->xsp_taglibs = NULL;
     new->current_styles = NULL;
     new->current_medias = NULL;
     new->error_stylesheet = NULL;
     new->output_transformers = NULL;
     new->current_plugins = NULL;
-
+    
 /*
     warn("[AxKit] created new dir_config:\n"
         "location: %d\n"
@@ -131,8 +127,7 @@ new_axkit_dir_config (pool *p)
         "new.reset_processors: %d\n"
         "new.cache_dir: %d\n"
         "new.config_reader_module: %d\n"
-        "new.contentprovider_module: %d\n"
-        "new.styleprovider_module: %d\n"
+        "new.provider_module: %d\n"
         "new.default_style: %d\n"
         "new.default_media: %d\n"
         "new.cache_module: %d\n"
@@ -140,6 +135,7 @@ new_axkit_dir_config (pool *p)
         "new.debug_level: %d\n"
         "new.type_map: %d\n"
         "new.processors: %d\n"
+        "new.dynamic_processors: %d\n"
         "new.xsp_taglibs: %d\n"
         "new.current_styles: %d\n"
         "new.current_medias: %d\n"
@@ -154,8 +150,7 @@ new_axkit_dir_config (pool *p)
         new->reset_processors,
         new->cache_dir,
         new->config_reader_module,
-        new->contentprovider_module,
-        new->styleprovider_module,
+        new->provider_module,
         new->default_style,
         new->default_media,
         new->cache_module,
@@ -163,6 +158,7 @@ new_axkit_dir_config (pool *p)
         new->debug_level,
         new->type_map,
         new->processors,
+        new->dynamic_processors,
         new->xsp_taglibs,
         new->current_styles,
         new->current_medias,
@@ -189,16 +185,19 @@ create_axkit_dir_config (pool *p, char *dummy)
     new->processors = newHV();
     ap_register_cleanup(p, (void*)new->processors, ax_cleanup_hv, ap_null_cleanup);
     
+    new->dynamic_processors = newAV();
+    ap_register_cleanup(p, (void*)new->dynamic_processors, ax_cleanup_av, ap_null_cleanup);
+    
     new->xsp_taglibs = newHV();
     ap_register_cleanup(p, (void*)new->xsp_taglibs, ax_cleanup_hv, ap_null_cleanup);
-
+    
     new->output_transformers = newAV();
     ap_register_cleanup(p, (void*)new->output_transformers, ax_cleanup_av, ap_null_cleanup);
     
     new->current_styles = newAV();
-    av_push(new->current_styles, newSVpv("#global", 0));
+    av_push(new->current_styles, newSVpv("#default", 0));
     ap_register_cleanup(p, (void*)new->current_styles, ax_cleanup_av, ap_null_cleanup);
-
+    
     new->current_medias = newAV();
     av_push(new->current_medias, newSVpv("screen", 0));
     ap_register_cleanup(p, (void*)new->current_medias, ax_cleanup_av, ap_null_cleanup);
@@ -208,9 +207,9 @@ create_axkit_dir_config (pool *p, char *dummy)
 
     new->current_plugins = newAV();
     ap_register_cleanup(p, (void*)new->current_plugins, ax_cleanup_av, ap_null_cleanup);
-
+    
     /* warn("create dir config: %d\n", new); */
-
+    
     return new;
 }
 
@@ -235,7 +234,7 @@ store_in_hv2 (HV * my_hv, SV * one, SV * two, SV * value)
         }
         sub1 = (HV*)SvRV(*sub1p);
     }
-
+    
     key2 = SvPV(two, len);
     if (!hv_exists(sub1, key2, len)) {
         sub2 = newAV();
@@ -248,9 +247,9 @@ store_in_hv2 (HV * my_hv, SV * one, SV * two, SV * value)
         }
         sub2 = (AV*)SvRV(*sub2p);
     }
-
+    
     /* warn("adding processor in %s/%s with refs: %d\n", key1, key2, SvREFCNT(value)); */
-
+    
     av_push(sub2, value);
 }
 
@@ -260,14 +259,14 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
     axkit_dir_config *parent_dir = (axkit_dir_config *)parent_dirv;
     axkit_dir_config *subdir = (axkit_dir_config *)subdirv;
     axkit_dir_config *new = new_axkit_dir_config(p);
-
+    
     /* Brian Wheeler found that sometimes parent is NULL */
     if (parent_dir == NULL) {
         parent_dir = create_axkit_dir_config(p, "");
     }
-
+    
     /* warn("merge : %d with %d\n", parent_dir, subdir); */
-
+    
 /*
     warn("[AxKit] merge: parent dir_config:\n"
         "location: %d\n"
@@ -278,8 +277,7 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
         "parent_dir.reset_processors: %d\n"
         "parent_dir.cache_dir: %d\n"
         "parent_dir.config_reader_module: %d\n"
-        "parent_dir.contentprovider_module: %d\n"
-        "parent_dir.styleprovider_module: %d\n"
+        "parent_dir.provider_module: %d\n"
         "parent_dir.default_style: %d\n"
         "parent_dir.default_media: %d\n"
         "parent_dir.cache_module: %d\n"
@@ -287,6 +285,7 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
         "parent_dir.debug_level: %d\n"
         "parent_dir.type_map: %d\n"
         "parent_dir.processors: %d\n"
+        "parent_dir.dynamic_processors: %d\n"
         "parent_dir.xsp_taglibs: %d\n"
         "parent_dir.current_styles: %d\n"
         "parent_dir.current_medias: %d\n"
@@ -301,8 +300,7 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
         parent_dir->reset_processors,
         parent_dir->cache_dir,
         parent_dir->config_reader_module,
-        parent_dir->contentprovider_module,
-        parent_dir->styleprovider_module,
+        parent_dir->provider_module,
         parent_dir->default_style,
         parent_dir->default_media,
         parent_dir->cache_module,
@@ -310,6 +308,7 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
         parent_dir->debug_level,
         parent_dir->type_map,
         parent_dir->processors,
+        parent_dir->dynamic_processors,
         parent_dir->xsp_taglibs,
         parent_dir->current_styles,
         parent_dir->current_medias,
@@ -326,8 +325,7 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
         "subdir.reset_processors: %d\n"
         "subdir.cache_dir: %d\n"
         "subdir.config_reader_module: %d\n"
-        "subdir.contentprovider_module: %d\n"
-        "subdir.styleprovider_module: %d\n"
+        "subdir.provider_module: %d\n"
         "subdir.default_style: %d\n"
         "subdir.default_media: %d\n"
         "subdir.cache_module: %d\n"
@@ -335,6 +333,7 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
         "subdir.debug_level: %d\n"
         "subdir.type_map: %d\n"
         "subdir.processors: %d\n"
+        "subdir.dynamic_processors: %d\n"
         "subdir.xsp_taglibs: %d\n"
         "subdir.current_styles: %d\n"
         "subdir.current_medias: %d\n"
@@ -349,8 +348,7 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
         subdir->reset_processors,
         subdir->cache_dir,
         subdir->config_reader_module,
-        subdir->contentprovider_module,
-        subdir->styleprovider_module,
+        subdir->provider_module,
         subdir->default_style,
         subdir->default_media,
         subdir->cache_module,
@@ -358,6 +356,7 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
         subdir->debug_level,
         subdir->type_map,
         subdir->processors,
+        subdir->dynamic_processors,
         subdir->xsp_taglibs,
         subdir->current_styles,
         subdir->current_medias,
@@ -373,42 +372,42 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
     else if (parent_dir->cache_dir) {
         new->cache_dir = ap_pstrdup(p, parent_dir->cache_dir);
     }
-
+    
     if (subdir->config_reader_module) {
         new->config_reader_module = ap_pstrdup(p, subdir->config_reader_module);
     }
     else if (parent_dir->config_reader_module) {
         new->config_reader_module = ap_pstrdup(p, parent_dir->config_reader_module);
     }
-
-    if (subdir->contentprovider_module) {
-        new->contentprovider_module = ap_pstrdup(p, subdir->contentprovider_module);
+    
+    if (subdir->provider_module) {
+        new->provider_module = ap_pstrdup(p, subdir->provider_module);
     }
-    else if (parent_dir->contentprovider_module) {
-        new->contentprovider_module = ap_pstrdup(p, parent_dir->contentprovider_module);
+    else if (parent_dir->provider_module) {
+        new->provider_module = ap_pstrdup(p, parent_dir->provider_module);
     }
-
+    
     if (subdir->styleprovider_module) {
         new->styleprovider_module = ap_pstrdup(p, subdir->styleprovider_module);
     }
     else if (parent_dir->styleprovider_module) {
         new->styleprovider_module = ap_pstrdup(p, parent_dir->styleprovider_module);
     }
-
+    
     if (subdir->default_style) {
         new->default_style = ap_pstrdup(p, subdir->default_style);
     }
     else if (parent_dir->default_style) {
         new->default_style = ap_pstrdup(p, parent_dir->default_style);
     }
-
+    
     if (subdir->default_media) {
         new->default_media = ap_pstrdup(p, subdir->default_media);
     }
     else if (parent_dir->default_media) {
         new->default_media = ap_pstrdup(p, parent_dir->default_media);
     }
-
+    
     if (subdir->cache_module) {
         new->cache_module = ap_pstrdup(p, subdir->cache_module);
     }
@@ -422,33 +421,15 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
     else if (parent_dir->output_charset) {
         new->output_charset = ap_pstrdup(p, parent_dir->output_charset);
     }
-
-    if (subdir->trace_intermediate) {
-        new->trace_intermediate = ap_pstrdup(p, subdir->trace_intermediate);
-    }
-    else if (parent_dir->trace_intermediate) {
-        new->trace_intermediate = ap_pstrdup(p, parent_dir->trace_intermediate);
-    }
-
-    if (subdir->dir_handler_module) {
-        new->dir_handler_module = ap_pstrdup(p, subdir->dir_handler_module);
-    }
-    else if (parent_dir->dir_handler_module) {
-        new->dir_handler_module = ap_pstrdup(p, parent_dir->dir_handler_module);
-    }
-
-    new->debug_tidy =
-        subdir->debug_tidy != -1 ? subdir->debug_tidy :
-                                     parent_dir->debug_tidy;
-
+    
     new->debug_level =
         subdir->debug_level != -1 ? subdir->debug_level :
                                     parent_dir->debug_level;
-
+    
     new->translate_output =
         subdir->translate_output != -1 ? subdir->translate_output :
                                          parent_dir->translate_output;
-
+    
     new->gzip_output =
         subdir->gzip_output != -1 ? subdir->gzip_output :
                                          parent_dir->gzip_output;
@@ -456,29 +437,22 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
     new->stack_trace =
         subdir->stack_trace != -1 ? subdir->stack_trace :
                                          parent_dir->stack_trace;
-
+    
     new->log_declines =
         subdir->log_declines != -1 ? subdir->log_declines :
                                      parent_dir->log_declines;
-
+    
     new->no_cache =
         subdir->no_cache != -1 ? subdir->no_cache :
                                      parent_dir->no_cache;
-
+    
     new->dependency_checks =
         subdir->dependency_checks != -1 ? subdir->dependency_checks :
                                      parent_dir->dependency_checks;
-
-    new->ignore_style_pi =
-        subdir->ignore_style_pi != -1 ? subdir->ignore_style_pi :
-                                    parent_dir->ignore_style_pi;
-
-    new->handle_dirs =
-        subdir->handle_dirs != -1 ? subdir->handle_dirs :
-                                    parent_dir->handle_dirs;
-
+    
+    
     /* complex types */
-
+    
     {
         /* cfg->error_stylesheet */
         AV * from = NULL;
@@ -496,7 +470,7 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
             char *stylesheet;
             SV ** avitem;
             STRLEN len;
-
+            
             avitem = av_fetch(from, 0, 0);
             mime = ap_pstrdup(p, SvPV(*avitem, len));
             av_push(new->error_stylesheet, newSVpvn(mime, strlen(mime)));
@@ -505,7 +479,7 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
             stylesheet = ap_pstrdup(p, SvPV(*avitem, len));
             av_push(new->error_stylesheet, newSVpvn(stylesheet, strlen(stylesheet)));
         }
-
+        
         ap_register_cleanup(p, (void*)new->error_stylesheet, ax_cleanup_av, ap_null_cleanup);
     }
     
@@ -532,22 +506,44 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
             cval = ap_pstrdup(p, SvPV(val, clen));
             hv_store(new->type_map, key, len, newSVpvn(cval, clen), 0);
         }
-
+        
         ap_register_cleanup(p, (void*)new->type_map, ax_cleanup_hv, ap_null_cleanup);
     }
-
+    
+    {
+        /* cfg->dynamic_processors */
+        new->dynamic_processors = newAV();
+        if (av_len(subdir->dynamic_processors) >= 0) {
+            I32 key = 0;
+            for(key = 0; key <= av_len(subdir->dynamic_processors); key++) {
+                SV ** val = av_fetch(subdir->dynamic_processors, key, 0);
+                if (val != NULL) {
+                    char * cval;
+                    STRLEN len;
+                    cval = ap_pstrdup(p, SvPV(*val, len));
+                    av_push(new->dynamic_processors, newSVpvn(cval, strlen(cval)));
+                }
+            }
+        }
+        else {
+            I32 key = 0;
+            for(key = 0; key <= av_len(parent_dir->dynamic_processors); key++) {
+                SV ** val = av_fetch(parent_dir->dynamic_processors, key, 0);
+                if (val != NULL) {
+                    char * cval;
+                    STRLEN len;
+                    cval = ap_pstrdup(p, SvPV(*val, len));
+                    av_push(new->dynamic_processors, newSVpvn(cval, strlen(cval)));
+                }
+            }
+        }
+        
+        ap_register_cleanup(p, (void*)new->dynamic_processors, ax_cleanup_av, ap_null_cleanup);
+    }
+    
     {
         /* cfg->xsp_taglibs */
         new->xsp_taglibs = newHV();
-        if (HvKEYS(parent_dir->xsp_taglibs)) {
-            SV * val;
-            char * key;
-            I32 len;
-            hv_iterinit(parent_dir->xsp_taglibs);
-            while (val = hv_iternextsv(parent_dir->xsp_taglibs, &key, &len)) {
-                hv_store(new->xsp_taglibs, key, len, newSViv(1), 0);
-            }
-        }
         if (HvKEYS(subdir->xsp_taglibs)) {
             SV * val;
             char * key;
@@ -557,17 +553,63 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
                 hv_store(new->xsp_taglibs, key, len, newSViv(1), 0);
             }
         }
-
+        else {
+            SV * val;
+            char * key;
+            I32 len;
+            hv_iterinit(parent_dir->xsp_taglibs);
+            while (val = hv_iternextsv(parent_dir->xsp_taglibs, &key, &len)) {
+                hv_store(new->xsp_taglibs, key, len, newSViv(1), 0);
+            }
+        }
+        
         ap_register_cleanup(p, (void*)new->xsp_taglibs, ax_cleanup_hv, ap_null_cleanup);
     }
-
+        
     {
         /* cfg->processors */
         SV * val;
         I32 len;
         char * key;
-
+        
         new->processors = newHV();
+        
+        if (!subdir->reset_processors) {
+            hv_iterinit(parent_dir->processors);
+            while (val = hv_iternextsv(parent_dir->processors, &key, &len)) {
+                SV * subval;
+                I32 sublen;
+                char * subkey;
+                HV * subhash = (HV*)SvRV(val);
+                
+                hv_iterinit(subhash);
+                while (subval = hv_iternextsv(subhash, &subkey, &sublen)) {
+                    SV * one;
+                    SV * two;
+                    AV * ary;
+                    I32 ary_len;
+                    I32 i;
+                    
+                    one = newSVpvn(ap_pstrdup(p, key), len);
+                    two = newSVpvn(ap_pstrdup(p, subkey), sublen);
+                    
+                    ary = (AV*)SvRV(subval);
+                    ary_len = av_len(ary);
+                    
+                    if (ary_len >= 0) {
+                        for (i = 0; i <= ary_len; i++) {
+                            SV ** elem = av_fetch(ary, i, 0);
+                            if (elem) {
+                                store_in_hv2(new->processors, one, two, newSVsv(*elem));
+                            }
+                        }
+                    }
+                    
+                    SvREFCNT_dec(one);
+                    SvREFCNT_dec(two);
+                }
+            }
+        }
 
         hv_iterinit(subdir->processors);
         while (val = hv_iternextsv(subdir->processors, &key, &len)) {
@@ -589,7 +631,7 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
                 
                 ary = (AV*)SvRV(subval);
                 ary_len = av_len(ary);
-
+                
                 if (ary_len >= 0) {
                     for (i = 0; i <= ary_len; i++) {
                         SV ** elem = av_fetch(ary, i, 0);
@@ -603,44 +645,7 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
                 SvREFCNT_dec(two);
             }
         }
-
-        if (!subdir->reset_processors) {
-            hv_iterinit(parent_dir->processors);
-            while (val = hv_iternextsv(parent_dir->processors, &key, &len)) {
-                SV * subval;
-                I32 sublen;
-                char * subkey;
-                HV * subhash = (HV*)SvRV(val);
-
-                hv_iterinit(subhash);
-                while (subval = hv_iternextsv(subhash, &subkey, &sublen)) {
-                    SV * one;
-                    SV * two;
-                    AV * ary;
-                    I32 ary_len;
-                    I32 i;
-
-                    one = newSVpvn(ap_pstrdup(p, key), len);
-                    two = newSVpvn(ap_pstrdup(p, subkey), sublen);
-
-                    ary = (AV*)SvRV(subval);
-                    ary_len = av_len(ary);
-
-                    if (ary_len >= 0) {
-                        for (i = 0; i <= ary_len; i++) {
-                            SV ** elem = av_fetch(ary, i, 0);
-                            if (elem) {
-                                store_in_hv2(new->processors, one, two, newSVsv(*elem));
-                            }
-                        }
-                    }
-                    
-                    SvREFCNT_dec(one);
-                    SvREFCNT_dec(two);
-                }
-            }
-        }
-
+        
         ap_register_cleanup(p, (void*)new->processors, ax_cleanup_hv, ap_null_cleanup);
     }
     
@@ -672,10 +677,10 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
                 }
             }
         }
-
+        
         ap_register_cleanup(p, (void*)new->output_transformers, ax_cleanup_av, ap_null_cleanup);
     }
-
+    
     {
         /* cfg->current_plugins */
         new->current_plugins = newAV();
@@ -704,14 +709,14 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
                 }
             }
         }
-
+        
         ap_register_cleanup(p, (void*)new->current_plugins, ax_cleanup_av, ap_null_cleanup);
     }
-
+    
     new->current_styles = newAV();
-    av_push(new->current_styles, newSVpv("#global", 0));
+    av_push(new->current_styles, newSVpv("#default", 0));
     ap_register_cleanup(p, (void*)new->current_styles, ax_cleanup_av, ap_null_cleanup);
-
+    
     new->current_medias = newAV();
     av_push(new->current_medias, newSVpv("screen", 0));
     ap_register_cleanup(p, (void*)new->current_medias, ax_cleanup_av, ap_null_cleanup);
@@ -720,6 +725,7 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
     warn("merge results: %d\n"
         "typemap: %d\n"
         "processors: %d\n"
+        "dynamic: %d\n"
         "taglibs: %d\n"
         "c_styles: %d\n"
         "c_medias: %d\n"
@@ -728,13 +734,14 @@ merge_axkit_dir_config (pool *p, void *parent_dirv, void *subdirv)
         new,
         new->type_map,
         new->processors,
+        new->dynamic_processors,
         new->xsp_taglibs,
         new->current_styles,
         new->current_medias,
         new->output_transformers
         );
 */
-
+    
     return new;
 }
 
@@ -745,15 +752,15 @@ ax_add_type_processor (cmd_parms *cmd, axkit_dir_config *ax,
                             char *mt, char *sty, char *option)
 {
     AV *processor = newAV();
-
+    
     SV **cur_media = av_fetch(ax->current_medias, 0, 0);
     SV **cur_style = av_fetch(ax->current_styles, 0, 0);
-
+    
     SV *type = newSVpv((char*)cmd->info, 0);
     SV *mime = newSVpv(mt, 0);
     SV *style = newSVpv(sty, 0);
     SV *opt_sv = newSVpv(option, 0);
-
+    
     av_push(processor, type);
     av_push(processor, mime);
     av_push(processor, style);
@@ -774,7 +781,10 @@ CHAR_P
 ax_add_dynamic_processor (cmd_parms *cmd, axkit_dir_config *ax,
                             char *module)
 {
-    return ax_add_type_processor(cmd, ax, "", "", module); 
+    SV * mod_sv = newSVpv(module, 0);
+    av_push(ax->dynamic_processors, mod_sv);
+    
+    return NULL;
 }
 
 CHAR_P
@@ -790,7 +800,7 @@ ax_error_stylesheet (cmd_parms *cmd, axkit_dir_config *ax, char *mime, char *sty
 {
     av_push(ax->error_stylesheet, newSVpvn(mime, strlen(mime)));
     av_push(ax->error_stylesheet, newSVpvn(stylesheet, strlen(stylesheet)));
-
+    
     return NULL;
 }
 
@@ -837,7 +847,7 @@ ax_media_type (cmd_parms *cmd, axkit_dir_config *ax, char *media)
     }
     
     ap_set_module_config(cmd->server->lookup_defaults, &XS_AxKit, oldconf);
-
+    
     return NULL;
 }
 
@@ -873,7 +883,7 @@ ax_style_name (cmd_parms *cmd, axkit_dir_config *ax, char *style)
 
     oldconf = ap_get_module_config(cmd->server->lookup_defaults, &XS_AxKit);
     ap_set_module_config(cmd->server->lookup_defaults, &XS_AxKit, (void*)ax);
-
+    
     while (!ap_cfg_getline(line, MAX_STRING_LEN, cmd->config_file)) {
         const char *errmsg;
         if (!strcasecmp(line, "</AxStyleName>")) {
@@ -929,13 +939,13 @@ CHAR_P
 ax_add_xsp_taglib (cmd_parms *cmd, axkit_dir_config *ax, char *module)
 {
     STRLEN len;
-
+    
     ax_preload_module(&module);
-
+    
     len = strlen(module);
-
+    
     hv_store(ax->xsp_taglibs, module, len, newSViv(1), 0);
-
+    
     return NULL;
 }
 
@@ -944,7 +954,7 @@ ax_add_output_transformer (cmd_parms *cmd, axkit_dir_config *ax, char *module)
 {
     SV * mod_sv = newSVpv(module, 0);
     av_push(ax->output_transformers, mod_sv);
-
+    
     return NULL;
 }
 
@@ -963,15 +973,6 @@ ax_add_plugin (cmd_parms *cmd, axkit_dir_config *ax, char *module)
     return NULL;
 }
 
-CHAR_P
-ax_set_provider (cmd_parms *cmd, axkit_dir_config *ax, char *provider)
-{
-    ax_preload_module(&provider);
-    ax->contentprovider_module = provider;
-    ax->styleprovider_module = provider;
-    return NULL;
-}
-
 CHAR_P
 ax_reset_plugins (cmd_parms *cmd, axkit_dir_config *ax)
 {
@@ -1014,169 +1015,31 @@ void axkit_module_init(server_rec *s, pool *p)
 static int axkit_handler(request_rec *r)
 {
     int retval;
-    SV * handler_sv;
-    SV * cfg;
-    SV * debuglevel;
-    SV * errorlevel;
+    SV * handler_sv = newSVpv("AxKit::fast_handler", 0);
+    
+    SV * cfg = perl_get_sv("AxKit::Cfg", FALSE);
+    SV * cache = perl_get_sv("AxKit::Cache", FALSE);
+    SV * hs = perl_get_sv("AxKit::HeadersSent", FALSE);
+    SV * debuglevel = perl_get_sv("AxKit::DebugLevel", FALSE);
+    SV * errorlevel = perl_get_sv("Error::Debug", FALSE);
     
-    if (S_ISDIR(r->finfo.st_mode)) {
-        axkit_dir_config * cfg = (axkit_dir_config *)ap_get_module_config(r->per_dir_config, &XS_AxKit);
-        if (!cfg || cfg->handle_dirs != 1) {
-            return DECLINED;
-        }
-    }
-
-    handler_sv = newSVpv("AxKit::fast_handler", 0);
-
-    cfg = perl_get_sv("AxKit::Cfg", FALSE);
-    debuglevel = perl_get_sv("AxKit::DebugLevel", FALSE);
-    errorlevel = perl_get_sv("Error::Debug", FALSE);
-
     ENTER;
-
+    
     save_item(cfg);
+    save_item(cache);
+    save_item(hs);
     save_item(debuglevel);
     save_item(errorlevel);
-
+    
     retval = perl_call_handler(handler_sv, (request_rec *)r, Nullav);
 
     LEAVE;
-
+    
     SvREFCNT_dec(handler_sv);
-
-    if (retval == DECLINED) {
-        r->handler = "default-handler";
-        return ap_invoke_handler(r);
-    }
-
+    
     return retval;
 }
 
-static int axkit_ok_handler(request_rec *r)
-{
-    return OK;
-}
-
-static int axkit_declined_handler(request_rec *r)
-{
-    return DECLINED;
-}
-
-/************ server wide configuration *******************/
-
-static axkit_server_config *
-new_axkit_server_config (pool *p)
-{
-    axkit_server_config *new =
-        (axkit_server_config *) ap_palloc(p, sizeof(axkit_server_config));
-
-    new->external_encoding = 0;
-    new->iconv_handle = 0;
-
-    return new;
-}
-
-void *
-create_axkit_server_config (pool *p, server_rec *s)
-{
-    axkit_server_config *new = new_axkit_server_config(p);
-
-    return new;
-}
-
-static void *
-merge_axkit_server_config (pool *p, void *parent_dirv, void *subdirv)
-{
-    axkit_server_config *parent_dir = (axkit_server_config *)parent_dirv;
-    axkit_server_config *subdir = (axkit_server_config *)subdirv;
-    axkit_server_config *new = new_axkit_server_config(p);
-
-    if (subdir->external_encoding) {
-        new->external_encoding = ap_pstrdup(p, subdir->external_encoding);
-        if (strcmp(new->external_encoding,"UTF-8")) {
-            new->iconv_handle = iconv_open(new->external_encoding,"UTF-8");
-            ap_register_cleanup(p, (void*)new->iconv_handle, (void (*)(void *))iconv_close, ap_null_cleanup);
-        }
-    }
-    else if (parent_dir && parent_dir->external_encoding) {
-        new->external_encoding = ap_pstrdup(p, parent_dir->external_encoding);
-        if (strcmp(new->external_encoding,"UTF-8")) {
-            new->iconv_handle = iconv_open(new->external_encoding,"UTF-8");
-            ap_register_cleanup(p, (void*)new->iconv_handle, (void (*)(void *))iconv_close, ap_null_cleanup);
-        }
-    }
-
-    return new;
-}
-
-static CHAR_P
-ax_set_external_encoding (cmd_parms *cmd, void *dummy, char *encoding)
-{
-    axkit_server_config *ax = (axkit_server_config *)
-                ap_get_module_config(cmd->server->module_config, &XS_AxKit);
-    /* warn("setting encoding %s",encoding); */
-    ax->external_encoding = ap_pstrdup(cmd->pool, encoding);
-    if (strcmp(ax->external_encoding,"UTF-8")) {
-        ax->iconv_handle = iconv_open(ax->external_encoding,"UTF-8");
-        ap_register_cleanup(cmd->pool, (void*)ax->iconv_handle, (void (*)(void *))iconv_close, ap_null_cleanup);
-    }
-    return NULL;
-}
-
-static int axkit_fixup_charsets(request_rec *r)
-{
-    char *local;
-    int inbytes, outbytes;
-    char *icursor, *ocursor;
-
-    axkit_server_config * cfg;
-    /* warn("fixup: '%s'",r->uri); */
-
-    /* see comments above */
-    if (!r || !r->server || !r->server->module_config)
-        return DECLINED;
-
-    cfg = (axkit_server_config *)ap_get_module_config(r->server->module_config, &XS_AxKit);
-
-    if (!cfg || !cfg->iconv_handle)
-        return DECLINED;
-
-    /* can "UTF-8 -> anything" grow by more than a factor 4?
-     * (UTF-8 -> UCS-4 could be factor 4, add some slack to be safe)
-     */
-    inbytes = strlen(r->uri);
-    outbytes = inbytes*4+12;
-    local = ap_pcalloc(r->pool, outbytes+1);
-
-    /* reset state */
-    iconv(cfg->iconv_handle,NULL,NULL,NULL,NULL);
-
-    icursor = r->uri;
-    ocursor = local;
-    /* Try conversion */
-    if (iconv(cfg->iconv_handle,&icursor,&inbytes,&ocursor,&outbytes) == (size_t)-1) {
-        /* conversion failed - assume it was sent in target charset */
-        /*warn("not UTF-8, leaving URL '%s' untouched", r->uri);*/
-        return DECLINED;
-    }
-
-    /*warn("conversion successful: '%s'",local);*/
-    r->uri = local;
-
-    return DECLINED;
-}
-
-void axkit_child_init(server_rec *s, pool *p)
-{
-    /* create server_config objects in advance, otherwise apache might try to
-    allocate one in a per-request pool, which is a remarkably dumb thing to do. */
-    while (s) {
-    	if (ap_get_module_config(s->module_config,&XS_AxKit) == NULL)
-    		ap_set_module_config(s->module_config,&XS_AxKit,create_axkit_server_config(p,s));
-    	s = s->next;
-    }
-}
-
 static command_rec axkit_mod_cmds[] = {
 
     { "AxAddProcessor", ax_add_processor,
@@ -1192,7 +1055,7 @@ static command_rec axkit_mod_cmds[] = {
       "a mime type, a stylesheet, and a dtd filename" },
 
     { "AxAddDynamicProcessor", ax_add_dynamic_processor,
-      (void*)"Dynamic", OR_ALL, TAKE1,
+      NULL, OR_ALL, TAKE1, 
       "a package name" },
 
     { "AxAddRootProcessor", ax_add_type_processor,
@@ -1216,7 +1079,7 @@ static command_rec axkit_mod_cmds[] = {
       "End of media type block" },
 
     { "<AxStyleName", ax_style_name,
-      NULL, OR_ALL, RAW_ARGS,
+      NULL, OR_ALL, RAW_ARGS, 
       "Style name block" },
 
     { "</AxStyleName>", ax_style_name_end,
@@ -1241,15 +1104,11 @@ static command_rec axkit_mod_cmds[] = {
       OR_ALL, TAKE1,
       "alternative module to use for reading configuration" },
 
-    { "AxContentProvider", ax_set_module_slot,
-      (void *)XtOffsetOf(axkit_dir_config, contentprovider_module),
+    { "AxProvider", ax_set_module_slot,
+      (void *)XtOffsetOf(axkit_dir_config, provider_module),
       OR_ALL, TAKE1,
       "alternative module to use for reading the xml" },
 
-    { "AxProvider", ax_set_provider,
-      NULL, OR_ALL, TAKE1,
-      "alternative module to use for reading the xml" },
-
     { "AxStyleProvider", ax_set_module_slot,
       (void *)XtOffsetOf(axkit_dir_config, styleprovider_module),
       OR_ALL, TAKE1,
@@ -1266,11 +1125,11 @@ static command_rec axkit_mod_cmds[] = {
       "a default media to use other than screen" },
 
     { "AxAddOutputTransformer", ax_add_output_transformer,
-      NULL, OR_ALL, TAKE1,
+      NULL, OR_ALL, TAKE1, 
       "An output transformer function, qualified with package name." },
 
     { "AxResetOutputTransformers", ax_reset_output_transformers,
-      NULL, OR_ALL, NO_ARGS,
+      NULL, OR_ALL, NO_ARGS, 
       "Reset list of output transformers." },
 
     { "AxCacheModule", ax_set_module_slot,
@@ -1292,16 +1151,6 @@ static command_rec axkit_mod_cmds[] = {
       OR_ALL, TAKE1,
       "character set used by iconv" },
 
-    { "AxTraceIntermediate", ap_set_string_slot,
-      (void *)XtOffsetOf(axkit_dir_config, trace_intermediate),
-      OR_ALL, TAKE1,
-      "location of a directory to write intermediate xml documents to (for debugging)" },
-
-    { "AxDebugTidy", ap_set_flag_slot,
-      (void *)XtOffsetOf(axkit_dir_config, debug_tidy),
-      OR_ALL, FLAG,
-      "On or Off [default] to tidy up source of debug output" },
-
     { "AxGzipOutput", ap_set_flag_slot,
       (void *)XtOffsetOf(axkit_dir_config, gzip_output),
       OR_ALL, FLAG,
@@ -1329,7 +1178,7 @@ static command_rec axkit_mod_cmds[] = {
     { "AxAddXSPTaglib", ax_add_xsp_taglib,
       NULL, OR_ALL, TAKE1,
       "module that provides a taglib functionality" },
-
+    
     { "AxDependencyChecks", ap_set_flag_slot,
       (void *)XtOffsetOf(axkit_dir_config, dependency_checks),
       OR_ALL, FLAG,
@@ -1343,33 +1192,12 @@ static command_rec axkit_mod_cmds[] = {
         NULL, OR_ALL, NO_ARGS,
         "reset the list of plugins" },
 
-    { "AxHandleDirs", ap_set_flag_slot,
-      (void *)XtOffsetOf(axkit_dir_config, handle_dirs),
-      OR_ALL, FLAG,
-      "On or Off [default] to make AxKit produce XML for directory requests" },
-
-    { "AxDirHandler", ax_set_module_slot,
-      (void *)XtOffsetOf(axkit_dir_config, dir_handler_module),
-      OR_ALL, TAKE1,
-      "alternative module to use for generating directory xml" },
-
-    { "AxIgnoreStylePI", ap_set_flag_slot,
-      (void *)XtOffsetOf(axkit_dir_config, ignore_style_pi),
-      OR_ALL, FLAG,
-      "On or Off [default] to disable xml-stylesheet PI processing" },
-
-    { "AxExternalEncoding", ax_set_external_encoding,
-      NULL,
-      RSRC_CONF, TAKE1,
-      "the character set used by the local file system and by URLs [default don't convert]" },
-
     { NULL }
 };
 
 static const handler_rec axkit_handlers[] =
 {
     {"axkit", axkit_handler},
-    { DIR_MAGIC_TYPE, axkit_handler },
     {NULL}
 };
 
@@ -1378,11 +1206,11 @@ module MODULE_VAR_EXPORT XS_AxKit = {
     axkit_module_init,            /* module initializer */
     create_axkit_dir_config,      /* per-directory config creator */
     merge_axkit_dir_config,       /* dir config merger */
-    create_axkit_server_config,   /* server config creator */
-    merge_axkit_server_config,    /* server config merger */
+    NULL,                         /* server config creator */
+    NULL,                         /* server config merger */
     axkit_mod_cmds,               /* command table */
     axkit_handlers,               /* [7] list of handlers */
-    axkit_fixup_charsets,         /* [2] filename-to-URI translation */
+    NULL,                         /* [2] filename-to-URI translation */
     NULL,                         /* [5] check/validate user_id */
     NULL,                         /* [6] check user_id is valid *here* */
     NULL,                         /* [4] check access by host address */
@@ -1390,7 +1218,7 @@ module MODULE_VAR_EXPORT XS_AxKit = {
     NULL,                         /* [8] fixups */
     NULL,                         /* [10] logger */
     NULL,                         /* [3] header parser */
-    axkit_child_init,                         /* process initializer */
+    NULL,                         /* process initializer */
     NULL,                         /* process exit/cleanup */
     NULL,                         /* [1] post read_request handling */
 };
@@ -1399,19 +1227,19 @@ HV *
 ax_get_config (axkit_dir_config * cfg)
 {
     HV * retval;
-
+    
     retval = newHV();
     if (cfg->cache_dir) {
-        hv_store(retval, "CacheDir",
+        hv_store(retval, "CacheDir", 
                 8, (newSVpv(cfg->cache_dir, 0)), 0);
     }
     if (cfg->config_reader_module) {
         hv_store(retval, "ConfigReader",
                 12, (newSVpv(cfg->config_reader_module, 0)), 0);
     }
-    if (cfg->contentprovider_module) {
-        hv_store(retval, "ContentProvider",
-                15, (newSVpv(cfg->contentprovider_module, 0)), 0);
+    if (cfg->provider_module) {
+        hv_store(retval, "Provider",
+                8, (newSVpv(cfg->provider_module, 0)), 0);
     }
     if (cfg->styleprovider_module) {
         hv_store(retval, "StyleProvider",
@@ -1433,14 +1261,6 @@ ax_get_config (axkit_dir_config * cfg)
         hv_store(retval, "OutputCharset",
                 13, (newSVpv(cfg->output_charset, 0)), 0);
     }
-    if (cfg->trace_intermediate) {
-        hv_store(retval, "TraceIntermediate",
-                17, (newSVpv(cfg->trace_intermediate, 0)), 0);
-    }
-    if (cfg->debug_tidy != -1) {
-        hv_store(retval, "DebugTidy",
-                9, (newSViv(cfg->debug_tidy)), 0);
-    }
     if (cfg->debug_level) {
         hv_store(retval, "DebugLevel",
                 10, (newSViv(cfg->debug_level)), 0);
@@ -1465,19 +1285,7 @@ ax_get_config (axkit_dir_config * cfg)
         hv_store(retval, "NoCache",
                 7, (newSViv(cfg->no_cache)), 0);
     }
-    if (cfg->ignore_style_pi != -1) {
-        hv_store(retval, "IgnoreStylePI",
-                13, (newSViv(cfg->ignore_style_pi)), 0);
-    }
-    if (cfg->handle_dirs != -1) {
-        hv_store(retval, "HandleDirs",
-                10, (newSViv(cfg->handle_dirs)), 0);
-    }
-    if (cfg->dir_handler_module) {
-        hv_store(retval, "DirHandler",
-                10, (newSVpv(cfg->dir_handler_module, 0)), 0);
-    }
-
+    
     if (cfg->dependency_checks != -1) {
         hv_store(retval, "DependencyChecks",
                 16, (newSViv(cfg->dependency_checks)), 0);
@@ -1486,7 +1294,7 @@ ax_get_config (axkit_dir_config * cfg)
         hv_store(retval, "DependencyChecks",
                 16, (newSViv(1)), 0);
     }
-
+    
     hv_store(retval, "OutputTransformers",
             18, newRV_inc((SV*)cfg->output_transformers), 0);
     hv_store(retval, "ErrorStylesheet",
@@ -1495,24 +1303,16 @@ ax_get_config (axkit_dir_config * cfg)
             8, newRV_inc((SV*)cfg->type_map), 0);
     hv_store(retval, "Processors",
             10, newRV_inc((SV*)cfg->processors), 0);
+    hv_store(retval, "DynamicProcessors",
+            17, newRV_inc((SV*)cfg->dynamic_processors), 0);
     hv_store(retval, "XSPTaglibs",
             10, newRV_inc((SV*)cfg->xsp_taglibs), 0);
     hv_store(retval, "Plugins",
             7, newRV_inc((SV*)cfg->current_plugins), 0);
-
+    
     return retval;
 }
 
-void
-ax_get_server_config (axkit_server_config * cfg, HV *retval)
-{
-    /* see comments above */
-    if (cfg->external_encoding) {
-        hv_store(retval, "ExternalEncoding",
-                16, (newSVpv(cfg->external_encoding, 0)), 0);
-    }
-}
-
 void remove_module_cleanup(void * ignore)
 {
     if (ap_find_linked_module(ap_find_module_name(&XS_AxKit))) {
@@ -1522,7 +1322,6 @@ void remove_module_cleanup(void * ignore)
     (void)hv_delete(GvHV(incgv), "AxKit.pm", 8, G_DISCARD);
 }
 
-
 /* Diff for styles being relative to .htaccess:
 Index: axconfig.c
 ===================================================================
@@ -1567,7 +1366,7 @@ diff -r1.2 axconfig.c
 >     else {
 >         style = newSVpv(sty, 0);
 >     }
->
+>     
 >     warn("style: %s\n", SvPV_nolen(style));
 >     
 674,675c703,725
