@@ -1,4 +1,4 @@
-# $Id: Exception.pm,v 1.17 2001/05/30 16:22:37 matt Exp $
+# $Id: Exception.pm,v 1.1 2002/01/13 20:45:11 matts Exp $
 
 package Apache::AxKit::Exception;
 use Error;
@@ -54,4 +54,8 @@ package Apache::AxKit::Exception::Retval;
 package Apache::AxKit::Exception::IO;
 @ISA = ('Apache::AxKit::Exception');
 
+# a hack to fix broken Error.pm 0.13 (fixed in 0.14)
+package Error;
+use overload bool => sub { 1 };
+
 1;
