@@ -1,4 +1,4 @@
-# $Id: XSP.pm,v 1.50 2001/06/04 13:27:31 matt Exp $
+# $Id: XSP.pm,v 1.8 2002/03/10 16:16:59 matts Exp $
 
 package Apache::AxKit::Language::XSP;
 
@@ -7,7 +7,6 @@ use Apache::AxKit::Language;
 use Apache::Request;
 use Apache::AxKit::Exception;
 use Apache::AxKit::Cache;
-use XML::Parser;
 
 use vars qw/@ISA/;
 
@@ -36,7 +35,7 @@ my $cache;
 
 sub handler {
     my $class = shift;
-    my ($r, $xml, undef) = @_;
+    my ($r, $xml, undef, $last_in_chain) = @_;
     
     _register_me_and_others();
     
@@ -108,13 +107,15 @@ sub handler {
         AxKit::Debug(5, 'Recompiling XSP script');
         AxKit::Debug(10, $to_eval);
         eval $to_eval;
-        AxKit::Debug(5, 'XSP Compilation finished');
         if ($@) {
             my $line = 1;
             $to_eval =~ s/\n/"\n".++$line." "/eg;
             warn("Script:\n1 $to_eval\n");
-            die "Failed to parse: $@";
+	        throw Apache::AxKit::Exception::Error(
+                -text => "Compilation failed: $@"
+            );
         }
+        AxKit::Debug(5, 'XSP Compilation finished');
     }
     
     no strict 'refs';
@@ -124,7 +125,7 @@ sub handler {
     
     $r->no_cache(1);
     
-    my $xsp_cache = Apache::AxKit::Cache->new($r, $package);
+    my $xsp_cache = Apache::AxKit::Cache->new($r, $package, $package->cache_params($r, $cgi));
     
     if (!$package->has_changed($xsp_cache->mtime()) && 
                 !$xml->has_changed($xsp_cache->mtime())) {
@@ -133,15 +134,24 @@ sub handler {
         return;
     }
     
-    eval {
-#        local $^W;
-        $r->pnotes('dom_tree', $cv->($r, $cgi));
-    };
+    my $dom = XML::LibXML::Document->createDocument("1.0", "UTF-8");
+    my $rc = eval { $cv->($r, $cgi, $dom); };
     if ($@) {
-        die "XSP Script failed: $@";
+    	die $@ if (ref($@));
+	    if ($to_eval) {
+        	my $line = 1;
+        	$to_eval =~ s/\n/"\n".++$line." "/eg;
+        	warn("Script:\n1 $to_eval\n");
+	    }
+	    throw Apache::AxKit::Exception::Error(
+        	-text => "Execution failed: $@"
+	    );
     }
-    
-    $xsp_cache->write( $r->pnotes('dom_tree')->toString );
+    $r->pnotes('dom_tree', $dom);
+    $r->print($dom->toString) if $last_in_chain;
+
+    $xsp_cache->write( $dom->toString );
+    return $rc;
 }
 
 sub register {
@@ -200,6 +210,12 @@ sub get_package_name {
     return "Apache::AxKit::Language::XSP::ROOT$filename";
 }
 
+sub makeSingleQuoted($) {
+    my $value = shift;
+    $value =~ s/([\\|])/\\$1/g;
+    return 'q|'.$value.'|';
+}
+
 ############################################################
 # SAX Handler code
 ############################################################
@@ -282,9 +298,11 @@ sub start_document {
     
     $e->{XSP_Script} = join("\n", 
                 "package $e->{XSP_Package}; \@$e->{XSP_Package}::ISA = ('Apache::AxKit::Language::XSP::Page');",
-                "#line 2 ".$e->{XSP_Line}."\n",
+                # "#line 2 ".$e->{XSP_Line}."\n", # This is wrong. Currently, line numbers are unrelated to the source file's lines.
+                                                  # Better leave it out, it makes debug output confusing.
                 "use Apache;",
-                "use XML::XPath;",
+                "use Apache::Constants qw(:common);",
+                "use XML::LibXML;",
                 );
     
     foreach my $ns (keys %Apache::AxKit::Language::XSP::tag_lib) {
@@ -313,7 +331,7 @@ sub end_document {
         }
     }
 
-    $e->{XSP_Script} .= "return \$document\n}\n";
+    $e->{XSP_Script} .= "return OK;\n}\n";
     
     return $e->{XSP_Script};
 }
@@ -454,7 +472,7 @@ sub comment {
             $e->{XSP_Script} .= $sub->($e, $comment);
         }
         elsif ($sub = $pkg->can("parse_comment")) {
-            $e->{XSP_Script} .= $sub->($e, $comment);
+            $e->{XSP_Script} .= $sub->($e, $comment->{Data});
         }
     }
 }
@@ -478,7 +496,7 @@ sub processing_instruction {
             $e->{XSP_Script} .= $sub->($e, $pi);
         }
         elsif ($sub = $pkg->can("parse_pi")) {
-            $e->{XSP_Script} .= $sub->($e, $pi);
+            $e->{XSP_Script} .= $sub->($e, $pi->{Target}, $pi->{Data});
         }
     }
 }
@@ -491,6 +509,8 @@ package AxKit::XSP::Core;
 
 use vars qw/@ISA $NS/;
 
+*makeSingleQuoted = \&Apache::AxKit::Language::XSP::makeSingleQuoted;
+
 @ISA = ('Apache::AxKit::Language::XSP');
 
 $NS = 'http://apache.org/xsp/core/v1';
@@ -532,20 +552,20 @@ sub characters {
 # output through a characters SAX event."
 
     if ($e->current_element() =~ /^(content)$/) {
-        $text =~ s/\|/\\\|/g;
+        $text = makeSingleQuoted($text);
 
         return <<"EOT";
 {
-    my \$text = XML::XPath::Node::Text->new(q|$text|);
-    \$parent->appendChild(\$text, 1); 
+    my \$text = \$document->createTextNode($text);
+    \$parent->appendChild(\$text); 
 }
 EOT
     }
     elsif ($e->current_element() =~ /^(attribute|comment|name)$/) {
         return '' if ($e->current_element() eq 'attribute' && !$e->{attrib_seen_name});
         $text =~ s/^\s*//; $text =~ s/\s*$//;
-        $text =~ s/\|/\\\|/g;    
-        return ". q|$text|";
+        $text = makeSingleQuoted($text);
+        return ". $text";
     }
     
 #    return '' unless $e->{XSP_User_Root};
@@ -590,14 +610,14 @@ sub start_element {
     elsif ($tag eq 'element') {
         if (my $name = $attribs{name}) {
             $e->manage_text(0);
-            return '{ my $elem = XML::XPath::Node::Element->new(q(' . $name . '));' .
-                    '$parent->appendChild($elem, 1); $parent = $elem; }' . "\n";
+            return '{ my $elem = $document->createElement(' . makeSingleQuoted($name) . ');' .
+                    '$parent->appendChild($elem); $parent = $elem; }' . "\n";
         }
     }
     elsif ($tag eq 'attribute') {
         if (my $name = $attribs{name}) {
             $e->{attrib_seen_name} = 1;
-            return '{ my $attr = XML::XPath::Node::Attribute->new(q|' . $name . '|, ""';
+            return '$parent->setAttribute('.makeSingleQuoted($name).', ""';
         }
         $e->{attrib_seen_name} = 0;
     }
@@ -607,10 +627,10 @@ sub start_element {
     elsif ($tag eq 'pi') {
     }
     elsif ($tag eq 'comment') {
-        return '{ my $comment = XML::XPath::Node::Comment->new(""';
+        return '{ my $comment = $document->createComment(""';
     }
     elsif ($tag eq 'text') {
-        return '{ my $text = XML::XPath::Node::Text->new(""';
+        return '{ my $text = $document->createTextNode(""';
     }
     elsif ($tag eq 'expr') {
 #        warn "expr: -2 = {", $node->{Parent}->{NamespaceURI}, "}", $node->{Parent}->{Name}, "\n";
@@ -618,7 +638,7 @@ sub start_element {
             if (!$e->manage_text() || $node->{Parent}->{Name} =~ /^(.*:)?content$/) {
                 return <<'EOT';
 {
-    my $text = XML::XPath::Node::Text->new(do {
+    my $text = $document->createTextNode("".do {
 EOT
             }
             elsif ($node->{Parent}->{Name} =~ /^(.*:)?(logic|expr)$/) {
@@ -628,7 +648,7 @@ EOT
         else {
             return <<'EOT';
 {
-    my $text = XML::XPath::Node::Text->new(do {
+    my $text = $document->createTextNode("".do {
 EOT
         }
         
@@ -664,17 +684,17 @@ sub end_element {
         return '$parent = $parent->getParentNode;' . "\n";
     }
     elsif ($tag eq 'attribute') {
-        return '); $parent->appendAttribute($attr, 1); }' . "\n";
+        return ');' . "\n";
     }
     elsif ($tag eq 'name') {
         if ($node->{Parent}->{Name} =~ /^(.*:)?element$/) {
             $e->manage_text(0, 1);
-            return '; my $elem = XML::XPath::Node::Element->new($name);' .
-                    '$parent->appendChild($elem, 1); $parent = $elem; }' . "\n";
+            return '; my $elem = $document->createElement($name);' .
+                    '$parent->appendChild($elem); $parent = $elem; }' . "\n";
         }
         elsif ($node->{Parent}->{Name} =~ /^(.*:)?attribute$/) {
             $e->{attrib_seen_name} = 1;
-            return '; my $attr = XML::XPath::Node::Attribute->new($name, ""';
+            return '; my $attr = $document->createAttribute($name, ""';
         }
         else {
             die "xsp:name parent node: $node->{Parent}->{Name} not valid";
@@ -683,10 +703,10 @@ sub end_element {
     elsif ($tag eq 'pi') {
     }
     elsif ($tag eq 'comment') {
-        return '); $parent->appendChild($comment, 1); }' . "\n";
+        return '); $parent->appendChild($comment); }' . "\n";
     }
     elsif ($tag eq 'text') {
-        return '); $parent->appendChild($text, 1); }' . "\n";
+        return '); $parent->appendChild($text); }' . "\n";
     }
     elsif ($tag eq 'expr') {
 #        warn "expr: -2 = {", $node->{Parent}->{NamespaceURI}, "}", $node->{Parent}->{Name}, "\n";
@@ -694,7 +714,7 @@ sub end_element {
             if (!$e->manage_text() || $node->{Parent}->{Name} =~ /^(.*:)?content$/) {
                 return <<'EOT';
 }); # xsp tag
-    $parent->appendChild($text, 1); 
+    $parent->appendChild($text); 
 }
 EOT
             }
@@ -705,7 +725,7 @@ EOT
         else {
             return <<'EOT';
 }); # non xsp tag
-    $parent->appendChild($text, 1); 
+    $parent->appendChild($text); 
 }
 EOT
         }
@@ -723,37 +743,91 @@ EOT
 
 package AxKit::XSP::DefaultHandler;
 
+*makeSingleQuoted = \&Apache::AxKit::Language::XSP::makeSingleQuoted;
+
 sub start_element {
     my ($e, $node) = @_;
     
+    my $code;
     if (!$e->{XSP_User_Root}) {
         $e->{XSP_Script} .= join("\n",
                 'sub handler {',
-                'my ($r, $cgi) = @_;',
-                'my $document = XML::XPath::Node::Element->new();',
+                'my ($r, $cgi, $document) = @_;',
                 'my ($parent);',
-                '$parent = $document;',
                 "\n",
                 );
         $e->{XSP_User_Root} = 1;
+#        $code = '{ my $elem = $document->createElement(q(' . $node->{Name} . '));' .
+#        $code = '{ my $elem = $document->createElementNS(q(' . ($node->{NamespaceURI} || "") . '), q(' . $node->{Name} . '));' .
+
+        $e->{XSP_Random_Prefix} = join("",map { ('a'..'z','A'..'Z','0'..'9','-','_')[rand(64)] } 1..5 );
+        $e->{XSP_Random_Sequence} = "aaaa";
+        $e->{XSP_Random_Map} = {};
+        if ($node->{NamespaceURI}) {
+	    	if ($node->{Name} !~ m/:/) {
+	    		if (exists $e->{XSP_Random_Map}{$node->{NamespaceURI}}) {
+				    $node->{Name} = $e->{XSP_Random_Map}{$node->{NamespaceURI}}.$e->{XSP_Random_Prefix}.':'.$node->{Name};
+	    		} 
+                else {
+				    $node->{Name} = $e->{XSP_Random_Sequence}.$e->{XSP_Random_Prefix}.':'.$node->{Name};
+				    $e->{XSP_Random_Map}{$node->{NamespaceURI}} = $e->{XSP_Random_Sequence};
+				    $e->{XSP_Random_Sequence}++;
+			    }
+	    	}
+		$code = '{ my $elem = $document->createElementNS('.makeSingleQuoted($node->{NamespaceURI}).','.makeSingleQuoted($node->{Name}).');' .
+                '$document->setDocumentElement($elem); $parent = $elem; }' . "\n";
+	    }
+        else {
+            $code = '{ my $elem = $document->createElement('.makeSingleQuoted($node->{Name}).');' .
+        	        '$document->setDocumentElement($elem); $parent = $elem; }' . "\n";
+	    }
+    }
+    else {
+#        $code = '{ my $elem = $document->createElement(q(' . $node->{Name} . '));' .
+        if ($node->{NamespaceURI}) {
+	    	if ($node->{Name} !~ m/:/) {
+	    		if (exists $e->{XSP_Random_Map}{$node->{NamespaceURI}}) {
+				    $node->{Name} = $e->{XSP_Random_Map}{$node->{NamespaceURI}}.$e->{XSP_Random_Prefix}.':'.$node->{Name};
+	    		} 
+                else {
+				    $node->{Name} = $e->{XSP_Random_Sequence}.$e->{XSP_Random_Prefix}.':'.$node->{Name};
+    				$e->{XSP_Random_Map}{$node->{NamespaceURI}} = $e->{XSP_Random_Sequence};
+	    			$e->{XSP_Random_Sequence}++;
+		    	}
+	    	}
+        	$code = '{ my $elem = $document->createElementNS('.makeSingleQuoted($node->{NamespaceURI}).','.makeSingleQuoted($node->{Name}).');' .
+                '$parent->appendChild($elem); $parent = $elem; }' . "\n";
+	    } 
+        else {
+        	$code = '{ my $elem = $document->createElement('.makeSingleQuoted($node->{Name}).');' .
+                '$parent->appendChild($elem); $parent = $elem; }' . "\n";
+        }
     }
     
-    my $code = '{ my $elem = XML::XPath::Node::Element->new(q(' . $node->{Name} . '));' .
-                '$parent->appendChild($elem, 1); $parent = $elem; }' . "\n";
-    
     for my $attr (@{$node->{Attributes}}) {
-        $code .= '{ my $attr = XML::XPath::Node::Attribute->new(q(' . $attr->{Name} . '), q(' . $attr->{Value} . '));';
-        $code .= '$parent->appendAttribute($attr, 1); }' . "\n";
+#        $code .= '$parent->setAttribute(q(' . $attr->{Name} . 
+#                '), q(' . $attr->{Value} . 
+#                '));' . "\n";
+        $code .= '$parent->setAttribute('.makeSingleQuoted($attr->{Name}).
+                ','.makeSingleQuoted($attr->{Value}).');' . "\n";
     }
 
     for my $ns (keys %{$e->{Current_NS}}) {
-        $code .= '{ my $ns = XML::XPath::Node::Namespace->new(q(' . $ns .'), q(' .
-                $e->{Current_NS}{$ns} . '));';
-        $code .= '$parent->appendNamespace($ns); }' . "\n";
+#        $code .= '$parent->setAttribute("xmlns:" . q(' . $ns .'), q(' .
+#                $e->{Current_NS}{$ns} . '));';
+    	if ($ns eq '#default') {
+	        $code .= '$parent->setAttribute("xmlns",' .
+        	        makeSingleQuoted($e->{Current_NS}{$ns}) . ');';
+      	} 
+        else {
+	        $code .= '$parent->setAttribute("xmlns:" . '.makeSingleQuoted($ns).',' .
+        	        makeSingleQuoted($e->{Current_NS}{$ns}) . ');';
+      	}
+
     }
     
     push @{ $e->{NS_Stack} },
-         { %{ $e->{Current_NS} } };
+            { %{ $e->{Current_NS} || {} } };
     
     $e->{Current_NS} = {};
     
@@ -781,8 +855,8 @@ sub characters {
     
     $text =~ s/\|/\\\|/g;
     
-    return '{ my $text = XML::XPath::Node::Text->new(q|' . $text . '|);' .
-            '$parent->appendChild($text, 1); }' . "\n";
+    return '{ my $text = $document->createTextNode('.makeSingleQuoted($text).');' .
+            '$parent->appendChild($text); }' . "\n";
 }
 
 sub comment {
@@ -796,15 +870,12 @@ sub processing_instruction {
 1;
 
 ######################################################
-## SAXParser - almost verbatim copy of Ken MacLeod's
-##             SAX2 stuff.
+## SAXParser
 ######################################################
 
 package AxKit::XSP::SAXParser;
 
-use vars qw/$xmlns_ns/;
-
-$xmlns_ns = "http://www.w3.org/2000/xmlns/";
+use XML::LibXML 1.30;
 
 sub new {
     my ($type, %self) = @_; 
@@ -814,128 +885,98 @@ sub new {
 sub parse {
     my ($self, $thing) = @_;
 
-    my $parser = XML::Parser->new( Handlers => {
-	Init => sub { $self->_handle_init(@_) },
-	Final => sub { $self->_handle_final(@_) },
-	Start => sub { $self->_handle_start(@_) },
-	End => sub { $self->_handle_end(@_) },
-	Char => sub { $self->_handle_char(@_) },
-	Comment => sub { $self->_handle_comment(@_) },
-	Proc => sub { $self->_handle_proc(@_) },
-    } );
+    my $doc;
+
+    my $parser = XML::LibXML->new();
+    $parser->expand_entities(1);
     
-    if ($self->{provider}) {
-        if (my $ext_ent_handler = $self->{provider}->get_ext_ent_handler()) {
-            $parser->setHandlers(ExternEnt => $ext_ent_handler);
+    if (ref($thing)) {
+        $doc = $parser->parse_fh($thing);
+    }
+    else {
+        $doc = $parser->parse_string($thing);
+    }
+    $doc->process_xinclude;
+    
+    my $encoding = $doc->getEncoding() || 'UTF-8';
+    my $document = { Parent => undef };
+    $self->{Handler}->start_document($document);
+    
+    my $root = $doc->getDocumentElement;
+    if ($root) {
+        process_node($self->{Handler}, $root, $encoding);
+    }
+    
+    $self->{Handler}->end_document($document);
+}
+
+sub process_node {
+    my ($handler, $node, $encoding) = @_;
+
+    
+    my $node_type = $node->getType();
+    if ($node_type == XML_COMMENT_NODE) {
+        $handler->comment( { Data => $node->getData } );
+    }
+    elsif ($node_type == XML_TEXT_NODE || $node_type == XML_CDATA_SECTION_NODE) {
+        # warn($node->getData . "\n");
+	    $handler->characters( { Data => encodeToUTF8($encoding,$node->getData()) } );
+    }
+    elsif ($node_type == XML_ELEMENT_NODE) {
+        # warn("<" . $node->getName . ">\n");
+        process_element($handler, $node, $encoding);
+        # warn("</" . $node->getName . ">\n");
+    }
+    elsif ($node_type == XML_ENTITY_REF_NODE) {
+        foreach my $kid ($node->getChildnodes) {
+            # warn("child of entity ref: " . $kid->getType() . " called: " . $kid->getName . "\n");
+            process_node($handler, $kid, $encoding);
         }
     }
-
-    $self->{InScopeNamespaceStack} = [ { '_Default' => undef,
-				         'xmlns' => $xmlns_ns } ];
-    $self->{NodeStack} = [ ];
-
-    return $parser->parse($thing);
-}
-
-sub _handle_init {
-    my ($self, $expat) = @_;
-
-    my $document = { Parent => undef };
-    push @{ $self->{NodeStack} }, $document;
-    $self->{Handler}->start_document( $document );
-}
-
-sub _handle_final {
-    my ($self, $expat) = @_;
-
-    my $document = pop @{ $self->{NodeStack} };
-    return $self->{Handler}->end_document( $document );
-}
-
-sub _handle_start {
-    my $self = shift; my $expat = shift; my $element_name = shift;
-
-    push @{ $self->{InScopeNamespaceStack} },
-         { %{ $self->{InScopeNamespaceStack}[-1] } };
-    $self->_scan_namespaces(@_);
-
-    my @attributes;
-    for (my $ii = 0; $ii < $#_; $ii += 2) {
-	my ($name, $value) = ($_[$ii], $_[$ii+1]);
-	my $namespace = $self->_namespace($name);
-	push @attributes, { Name => $name,
-                            Value => $value,
-                            NamespaceURI => $namespace };
+    elsif ($node_type == XML_DOCUMENT_NODE) {
+        # just get root element. Ignore other cruft.
+        foreach my $kid ($node->getChildnodes) {
+            if ($kid->getType() == XML_ELEMENT_NODE) {
+                process_element($handler, $kid, $encoding);
+                last;
+            }
+        }
     }
-
-    my $namespace = $self->_namespace($element_name);
-    my $element = { Name => $element_name,
-                    NamespaceURI => $namespace,
-                    Attributes => [ @attributes ],
-                    Parent => $self->{NodeStack}[-1] };
-
-    push @{ $self->{NodeStack} }, $element;
-    $self->{Handler}->start_element( $element );
-}
-
-sub _handle_end {
-    my $self = shift;
-
-    pop @{ $self->{InScopeNamespaceStack} };
-    my $element = pop @{ $self->{NodeStack} };
-    my $results = $self->{Handler}->end_element( $element );
-    return $results;
-}
-
-sub _handle_char {
-    my ($self, $expat, $string) = @_;
-
-    my $characters = { Data => $string, Parent => $self->{NodeStack}[-1] };
-    $self->{Handler}->characters( $characters );
-}
-
-sub _handle_comment {
-    my ($self, $expat, $data) = @_;
-
-    my $comment = { Data => $data, Parent => $self->{NodeStack}[-1] };
-    $self->{Handler}->comment( $comment );
-}
-
-sub _handle_proc {
-    my ($self, $expat, $target, $data) = @_;
-
-    my $pi = {  Target => $target,
-                Data => $data,
-                Parent => $self->{NodeStack}[-1] };
-    $self->{Handler}->processing_instruction( $pi );
-}
-
-sub _scan_namespaces {
-    my ($self, %attributes) = @_;
-
-    while (my ($attr_name, $value) = each %attributes) {
-	if ($attr_name eq 'xmlns') {
-	    $self->{InScopeNamespaceStack}[-1]{'_Default'} = $value;
-	} elsif ($attr_name =~ /^xmlns:(.*)$/) {
-	    my $prefix = $1;
-	    $self->{InScopeNamespaceStack}[-1]{$prefix} = $value;
-	}
+    else {
+        warn("unknown node type: $node_type");
     }
 }
 
-sub _namespace {
-    my ($self, $name) = @_;
-
-    my ($prefix, $localname) = split(/:/, $name);
-    if (!defined($localname)) {
-	if ($prefix eq 'xmlns') {
-	    return undef;
-	} else {
-	    return $self->{InScopeNamespaceStack}[-1]{'_Default'};
-	}
-    } else {
-	return $self->{InScopeNamespaceStack}[-1]{$prefix};
+sub process_element {
+    my ($handler, $element, $encoding) = @_;
+    
+    my @attr;
+    
+    foreach my $attr ($element->getAttributes) {
+        push @attr, {
+            Name => encodeToUTF8($encoding,$attr->getName),
+            Value => encodeToUTF8($encoding,$attr->getData),
+            NamespaceURI => encodeToUTF8($encoding,$attr->getNamespaceURI),
+            Prefix => encodeToUTF8($encoding,$attr->getPrefix),
+            LocalName => encodeToUTF8($encoding,$attr->getLocalName),
+        };
     }
+    
+    my $node = {
+        Name => encodeToUTF8($encoding,$element->getName),
+        Attributes => \@attr,
+        NamespaceURI => encodeToUTF8($encoding,$element->getNamespaceURI),
+        Prefix => encodeToUTF8($encoding,$element->getPrefix),
+        LocalName => encodeToUTF8($encoding,$element->getLocalName),
+    };
+    
+    $handler->start_element($node);
+    
+    foreach my $child ($element->getChildnodes) {
+        process_node($handler, $child, $encoding);
+    }
+    
+    $handler->end_element($node);
 }
 
 ############################################################
@@ -950,6 +991,12 @@ sub has_changed {
     return 1;
 }
 
+sub cache_params {
+    my $class = shift;
+    my ($r, $cgi) = @_;
+    return '';
+}
+
 1;
 
 __END__
@@ -996,6 +1043,29 @@ way of providing an environment for dynamic pages. XSP is originally part
 of the Apache Cocoon project, and so you will see some Apache namespaces
 used in XSP.
 
+A warning to namespace users: Do not expect your namespace _prefixes_ to
+come out of an XSP transformation as they were fed in. But since you are using
+namespaces, this doesn't really matter. You just have to make sure that
+each and every step in your transformation process is namespaces aware
+and uses the correct namespace declarations.
+
+=head2 Result Code
+
+You can specify the result code of the request in two ways. Both actions
+go inside a <xsp:logic> tag.
+
+If you want to completely abort the current request, throw an exception:
+
+	throw Apache::AxKit::Exception::Retval(return_code => FORBIDDEN);
+
+If you want to send your page but have a custom result code, return it:
+
+	return FORBIDDEN;
+
+In that case, only the part of the document that was processed so far gets
+sent/processed further.
+
+
 =head1 Tag Reference
 
 =head2 C<<xsp:page>>
@@ -1143,18 +1213,31 @@ Needless to say, in every case, C<<xsp:expr>> should just "do the right
 thing". If it doesn't, then something (either a taglib or XSP.pm itself)
 is broken and you should report a bug.
 
-=head1 DESIGN PATTERNS
+=head1 Writing Taglibs
 
 Writing your own taglibs can be tricky, because you're using an event
 based API to write out Perl code. You may want to take a look at the
 Apache::AxKit::Language::XSP::TaglibHelper module, which comes with
 AxKit and allows you to easily publish a taglib without writing
-XML event code.
+XML event code. Recently, another taglib helper has been developed,
+Apache::AxKit::Language::XSP::SimpleTaglib. The latter manages all the
+details described under 'Design Patterns' for you, so you don't really
+need to bother with them anymore.
+
+A warning about character sets: All string values are passed in and
+expected back as UTF-8 encoded strings. So you cannot use national characters
+in a different encoding, like the widespread ISO-8859-1. This applies to
+Taglib source code only. The XSP XML-source is of course interpreted according
+to the XML rules. Your taglib module may want to 'use utf8;' as well, see
+L<perlunicode> and L<utf8> for more information.
+
+=head1 Design Patterns
+
 
 These patterns represent the things you may want to achieve when 
 authoring a tag library "from scratch".
 
-B<1. Your tag is a wrapper around other things.>
+=head2 1. Your tag is a wrapper around other things.
 
 Example:
 
@@ -1186,7 +1269,7 @@ in parse_end:
 
 Note there the closing of that original opening block.
 
-B<2. Your tag indicates a parameter for a surrounding taglib.>
+=head2 2. Your tag indicates a parameter for a surrounding taglib.
 
 Example:
 
@@ -1212,8 +1295,8 @@ sub parse_char {
 
   return '' unless $text;
 
-  $text =~ s/\|/\\\|/g;
-  return ". q|$text|";
+  $text = Apache::AxKit::Language::XSP::makeSingleQuoted($text);
+  return ". $text";
 }
 
 Note there's no semi-colon at the end of all this, so we add that:
@@ -1227,9 +1310,11 @@ in parse_end:
 All of this black magic allows other taglibs to set the thing in that
 variable using expressions.
 
-B<3. You want your tag to return a scalar (string) that does the right thing
-depending on context. For example, generates a Text node in one place or
-generates a scalar in another context.>
+=head2 3. You want your tag to return a scalar (string) that does the right thing
+depending on context. 
+
+For example, generates a Text node in one place or generates a scalar in another 
+context.
 
 Solution:
 
@@ -1314,7 +1399,7 @@ by default, so you need to do:
   use Apache::AxKit::Language::XSP 
         qw(start_expr end_expr append_to_script);
 
-B<4. Your tag can take as an option either an attribute, or a child tag.>
+=head2 4. Your tag can take as an option either an attribute, or a child tag.
 
 Example:
 
@@ -1369,8 +1454,7 @@ in parse_end:
     ';
   }
 
-B<5. You want to return a scalar that does the right thing in context, but
-also can take a parameter as an attribute I<or> a child tag.>
+=head2 5. You want to return a scalar that does the right thing in context, but also can take a parameter as an attribute I<or> a child tag.
 
 Example:
 
@@ -1412,7 +1496,7 @@ in parse_end:
     return '';
   }
 
-B<6. You have a conditional tag>
+=head2 6. You have a conditional tag
 
 Example:
 
@@ -1437,7 +1521,7 @@ text nodes of this tag don't fire text events to the tag library, but
 instead get handled by XSP core, thus creating text nodes (and doing
 the right thing, generally).
 
-=head1 <xsp:expr> (and start_expr, end_expr) Notes
+=head2 <xsp:expr> (and start_expr, end_expr) Notes
 
 B<Do not> consider adding in the 'do {' ... '}' bits yourself. Always
 leave this to the start_expr, and end_expr functions. This is because the
