@@ -1,9 +1,10 @@
-# $Id: Provider.pm,v 1.33 2001/06/04 16:00:33 matt Exp $
+# $Id: Provider.pm,v 1.3 2002/03/15 13:35:04 matts Exp $
 
 package Apache::AxKit::Provider;
 use strict;
 
 use Apache::AxKit::Exception;
+use Apache::Constants qw(OK DECLINED);
 #use XML::Parser;
 
 # use vars qw/$COUNT/;
@@ -45,10 +46,9 @@ sub has_changed {
 
 sub decline {
     my $self = shift;
-    my %args = @_;
     
-    throw Apache::AxKit::Exception::Declined(
-                reason => $args{reason} || "provider declined");
+    AxKit::Debug(4, "provider declined");
+    return DECLINED;
 }
 
 sub get_ext_ent_handler {
@@ -64,8 +64,9 @@ sub get_ext_ent_handler {
                 require HTTP::GHTTP;
             };
             if ($@) {
-                require LWP;
-                return XML::Parser::lwp_ext_ent_handler(@_);
+                require LWP::Simple;
+                import LWP::Simple;
+                return get($sysid) || die "Cannot get $sysid";
             }
             my $r = HTTP::GHTTP->new($sysid);
             $r->process_request;
@@ -111,7 +112,7 @@ sub get_styles {
     # use three element array @$vals
     
     if (defined &Apache::AxKit::Provider::xs_get_styles_fh) {
-        AxKit::Debug(2, "using XS get_styles (libxml2)\n");
+        AxKit::Debug(2, "using XS get_styles (libxml2)");
         my ($xs_styles, $doctype, $dtd, $root) = 
                 $self->xs_get_styles($media, $pref_style);
         @$xml_styles = @$xs_styles;
@@ -153,6 +154,10 @@ sub get_styles {
         AxKit::Debug(4, "get_styles: parse returned successfully");
     }
     
+    foreach my $style (@$xml_styles) {
+        $style->{title} ||= '#default';
+    }
+    
     # Let GetMatchingProcessors to process the @$styles array
     AxKit::Debug(4, "Calling GetMatchingProcessors with ($media, $pref_style, $vals->[0], $vals->[1], $vals->[2])");
     my @styles = $AxKit::Cfg->GetMatchingProcessors($media,
@@ -200,12 +205,12 @@ sub xs_get_styles {
     my $bits;
     eval {
         my $fh = $self->get_fh();
-        AxKit::Debug(4, "calling xs_get_styles_fh()\n");
+        AxKit::Debug(4, "calling xs_get_styles_fh()");
         $bits = xs_get_styles_fh($self->apache_request, $fh);
     };
     if ($@) {
         my $strref = $self->get_strref();
-        AxKit::Debug(4, "calling xs_get_styles_str()\n");
+        AxKit::Debug(4, "calling xs_get_styles_str()");
         $bits = xs_get_styles_str($self->apache_request, $$strref);
     }
     
@@ -246,7 +251,7 @@ sub xs_get_styles {
         unshift @{$e->{XMLStyle_style}}, @{$e->{XMLStyle_style_persistant}};
     }
     
-    AxKit::Debug(4, "xs_get_styles returned: $bits->[3], $bits->[4], $element\n");
+    AxKit::Debug(4, "xs_get_styles returned: $bits->[3], $bits->[4], $element");
     
     return ($e->{XMLStyle_style}, $bits->[3], $bits->[4], $element);
 }
