@@ -1,4 +1,4 @@
-# $Id: AxKit.pm,v 1.39.2.4 2003/02/08 15:56:16 matts Exp $
+# $Id: AxKit.pm,v 1.3 2002/03/15 13:37:52 matts Exp $
 
 package AxKit;
 use strict;
@@ -17,13 +17,11 @@ use Apache::AxKit::Provider::File;
 use Apache::AxKit::Provider::Scalar;
 use Apache::AxKit::CharsetConv;
 use File::Basename ();
-use Compress::Zlib ();
-use Fcntl;
 
 Apache::AxKit::CharsetConv::raise_error(1);
 
 BEGIN {
-    $VERSION = "1.61";
+    $VERSION = "1.51";
     if ($ENV{MOD_PERL}) {
         $AxKit::ServerString = "AxKit/$VERSION";
         @AxKit::ISA = qw(DynaLoader);
@@ -35,27 +33,6 @@ BEGIN {
 # AxKit Utility Functions
 ###############################################################
 
-sub FromUTF8($) {
-    if (!$AxKit::Cfg->{from_utf8}) {
-        return $_[0] if (exists $AxKit::Cfg->{from_utf8});
-        AxKit::Debug(9,"encoding to ".$AxKit::Cfg->ExternalEncoding());
-        $AxKit::Cfg->{from_utf8} = undef, return $_[0] if ($AxKit::Cfg->ExternalEncoding() eq "UTF-8");
-        $AxKit::Cfg->{from_utf8} = Apache::AxKit::CharsetConv->new("UTF-8",$AxKit::Cfg->ExternalEncoding()) || die "ExternalEncoding: No such charset: ".$AxKit::Cfg->ExternalEncoding();
-    }
-    return $AxKit::Cfg->{from_utf8}->convert($_[0]);
-}
-
-sub ToUTF8($) {
-    if (!$AxKit::Cfg->{to_utf8}) {
-        return $_[0] if (exists $AxKit::Cfg->{to_utf8});
-        AxKit::Debug(9,"decoding from ".$AxKit::Cfg->ExternalEncoding());
-        $AxKit::Cfg->{to_utf8} = undef, return $_[0] if ($AxKit::Cfg->ExternalEncoding() eq "UTF-8");
-        $AxKit::Cfg->{to_utf8} = Apache::AxKit::CharsetConv->new($AxKit::Cfg->ExternalEncoding(), "UTF-8") || die "ExternalEncoding: No such charset: ".$AxKit::Cfg->ExternalEncoding();
-    }
-    return $AxKit::Cfg->{to_utf8}->convert($_[0]);
-}
-
-
 sub _Debug {
     my $level = shift;
     if ($level <= $AxKit::DebugLevel) {
@@ -114,7 +91,7 @@ sub get_output_transformer {
             return map { $map->convert( $_ ) } ($outputfunc->(@_));
         };
     }
-
+    
     foreach my $AxOutputTransformer ( $AxKit::Cfg->OutputTransformers() ) {
         $actually_transform = 1;
         my $outputfunc = $func;
@@ -123,7 +100,7 @@ sub get_output_transformer {
             map { &{$AxOutputTransformer}( $_ ) } ($outputfunc->(@_));
         };
     }
-
+    
     # to add a new output_transformer here:
     #   enter new scope (maybe with if())
     #   copy $func to a new lexical (my) variable
@@ -158,13 +135,8 @@ sub get_depends {
 
 sub fast_handler {
     my $r = shift;
-
+    
     local $SIG{__DIE__} = sub { AxKit::prep_exception(@_)->throw };
-
-    # use Carp ();
-    # local $SIG{'USR2'} = sub { 
-    #     Carp::confess("caught SIGUSR2!");
-    # };
     
     $AxKit::Cfg = Apache::AxKit::ConfigReader->new($r);
 
@@ -175,9 +147,7 @@ sub fast_handler {
 
     $Error::Debug = 1 if (($AxKit::Cfg->DebugLevel() > 3) || $AxKit::Cfg->StackTrace);
 
-    AxKit::Debug(1, "fast handler called for " . $r->uri);
-    
-    local $AxKit::FastHandler = 1;
+    AxKit::Debug(1, "handler called for " . $r->uri);
 
     my $plugin_ret = AxKit::run_plugins($r);
     if ($plugin_ret != OK) {
@@ -185,10 +155,10 @@ sub fast_handler {
         return $plugin_ret;
     }
 
-    my $provider = Apache::AxKit::Provider->new_content_provider($r);
+    my $provider = Apache::AxKit::Provider->new($r);
 
     return $provider->decline(reason => "passthru set")
-            if ($r->notes('axkit_passthru') && $r->dir_config('AxFastPassthru'));
+            if $r->notes('axkit_passthru');
 
     return main_handler($r, $provider);
 }
@@ -210,13 +180,10 @@ sub handler {
 #     ##############################
 
     local $SIG{__DIE__} = sub { AxKit::prep_exception(@_)->throw };
-
-    # use Carp ();
-    # local $SIG{'USR2'} = sub {
-    #     Carp::confess("caught SIGUSR2!");
-    # };
-
+    
     local $AxKit::Cfg;
+    local $AxKit::Cache;
+    local $AxKit::HeadersSent;
     local $AxKit::DebugLevel;
     local $Error::Debug;
 
@@ -227,65 +194,125 @@ sub handler {
         $AxKit::T0 = [Time::HiRes::gettimeofday()] if $AxKit::Cfg->DebugTime;
     }
 
-    $Error::Debug = 1 if (($AxKit::Cfg->DebugLevel() > 3) || $AxKit::Cfg->StackTrace);
+    $AxKit::DebugLevel = $AxKit::Cfg->DebugLevel();
+    $Error::Debug = 1 if (($AxKit::DebugLevel > 3) || $AxKit::Cfg->StackTrace);
 
     AxKit::Debug(1, "handler called for " . $r->uri);
 
-    local $AxKit::FastHandler = 0;
-
     my $plugin_ret = AxKit::run_plugins($r);
     if ($plugin_ret != OK) {
         AxKit::Debug(2, "Plugin returned non-OK value");
         return $plugin_ret;
     }
 
-    my $provider = Apache::AxKit::Provider->new_content_provider($r);
+    my $provider = Apache::AxKit::Provider->new($r);
 
     return $provider->decline(reason => "passthru set")
-            if ($r->notes('axkit_passthru') && $r->dir_config('AxFastPassthru'));
+            if $r->notes('axkit_passthru');
 
+    # Do we process this URL?
+    AxKit::Debug(2, "checking if we process this resource");
+    if (!$provider->process()) {
+        return $provider->decline();
+    }
+    
     return main_handler($r, $provider);
 }
 
 sub main_handler {
     my ($r, $provider) = @_;
-
-    # Do we process this URL?
-    # (moved down here from slow_handler because of AxHandleDirs)
-    AxKit::Debug(2, "checking if we process this resource");
-    if (!$provider->process()) {
-        return $provider->decline();
-    }
-
-    # setup global pnotes.
-    local $AxKit::Apache::PNOTES = $r->pnotes();
     
-    if ($r->notes('axkit_passthru')) {
-        # slow passthru
-        $r->send_http_header('text/xml');
-        eval {
-            my $fh = $provider->get_fh;
-            $r->send_fd($fh);
-        };
-        if ($@) {
-            my $str = $provider->get_strref;
-            $r->print($str);
-        }
-        return OK;
-    }
-
-    local $AxKit::Cache;
-
     my $retcode = eval {
         # $r->header_out('X-AxKit-Version', $VERSION);
-
+        
         chdir(File::Basename::dirname($r->filename));
-
+        
         $AxKit::OrigType = $r->content_type('changeme');
+        
+        # get preferred stylesheet and media type
+        my ($preferred, $media) = get_style_and_media();
+        AxKit::Debug(2, "media: $media, preferred style: $preferred");
+        
+        # get cache object
+        my $cache = Apache::AxKit::Cache->new($r, $r->filename() . '.gzip' . ($ENV{PATH_INFO} || ''), $preferred, $media, $r->notes('axkit_cache_extra'));
+        
+        my $recreate; # regenerate from source (not cached)
+        
+        my $styles = get_styles($media, $preferred, $cache, $provider);
+        
+        {
+            local $^W;
+            if ($preferred && ($styles->[0]{title} ne $preferred)) {
+                # we selected a style that didn't exist.
+                # Make sure we default the cache file, otherwise
+                # we setup a potential DoS
+                AxKit::Debug(3, "resetting cache with no preferred style ($preferred ne $styles->[0]{title})");
+                $cache = Apache::AxKit::Cache->new($r, $r->filename() . '.gzip' . $ENV{PATH_INFO}, '', $media, $r->notes('axkit_cache_extra'));
+            }
+        }
+        
+        if (!$cache->exists()) {
+            AxKit::Debug(2, "cache doesn't exist");
+            # set no_cache header if cache doesn't exist due to no_cache option
+            $r->no_cache(1) if $cache->no_cache();
+            $recreate++;
+        }
+        
+        if (!$recreate && $AxKit::Cfg->DependencyChecks()) {
+            $recreate = check_dependencies($r, $provider, $cache);
+        }
+
+        if (!$recreate && $r->method() eq 'POST') {
+            $recreate++;
+        }
+
+        $AxKit::Charset = $AxKit::Cfg->OutputCharset();
+
+        if (!$recreate) {
+            AxKit::Debug(1, "delivering cached copy - all conditions met");
+            return $cache->deliver();
+            AxKit::Debug(1, "UNREACHABLE CODE!!!");
+        }
+
+        AxKit::Debug(1, "some condition failed. recreating output");
+
+        # Store in package variable for other modules
+        $AxKit::Cache = $cache;
+
+        # reconsecrate Apache request object (& STDOUT) into our own class
+        bless $r, 'AxKit::Apache';
+        tie *STDOUT, 'AxKit::Apache', $r;
+
+        if (my $charset = $AxKit::Cfg->OutputCharset) {
+            AxKit::Debug(5, "Different output charset: $charset");
+            if (!$r->notes('axkit_passthru_type')) {
+                $r->content_type("text/html; charset=$charset");
+            }
+        }
+        
+        my $uri = $r->uri();
+        my $path_info = $r->path_info();
+        $uri =~ s/\Q$path_info\E$//;
+        $r->uri($uri);
 
         reset_depends();
 
-        my $result_code = run_axkit_engine($r, $provider);
+        {
+            # copy styles because we blat the copy
+            my @copy = @$styles;
+            $AxKit::_CurrentStylesheets = \@copy;
+        } 
+        # Main grunt of the work done here...
+        my $result_code = process_request($r, $provider, $AxKit::_CurrentStylesheets);
+
+        save_dependencies($r, $cache);
+        
+#        if (my $dom = $r->pnotes('dom_tree')) {
+#            AxKit::Debug(4, "got a dom_tree back - outputting that to the cache");
+#            $r->notes('resetstring', 1);
+#            my $output = $dom->toString;
+#            $r->print($output);
+#        }
 
         # restore $r
         if (ref($r) eq 'AxKit::Apache') {
@@ -307,7 +334,7 @@ sub main_handler {
         bless $r, 'Apache';
         tie *STDOUT, 'Apache', $r;
     }
-
+    
     if ($E->isa('Apache::AxKit::Exception::OK')) {
         return deliver_to_browser($r); # should return OK
     }
@@ -326,22 +353,7 @@ sub main_handler {
                     if $E->{reason};
         }
         AxKit::Debug(4, "[DECLINED] From: $E->{-file} : $E->{-line}");
-        
-        $r->send_http_header('text/xml');
-        eval {
-            my $fh = $provider->get_fh;
-            $r->send_fd($fh);
-        };
-        if ($@) {
-            eval {
-                my $str = $provider->get_strref;
-                $r->print($str);
-            };
-            if ($@) {
-                return DECLINED;
-            }
-        }
-        return OK;
+        return DECLINED;
     }
     elsif ($E->isa('Apache::AxKit::Exception::Error')) {
         $r->log->error("[AxKit] [Error] $E->{-text}");
@@ -359,7 +371,7 @@ sub main_handler {
         return SERVER_ERROR;
 
     }
-    elsif ($E->isa('Apache::AxKit::Exception')) {
+    elsif ($E->isa('Error::Simple') || $E->isa('Apache::AxKit::Exception')) {
         $r->log->error("[AxKit] [UnCaught] $E");
 
         if ($Error::Debug) {
@@ -380,214 +392,6 @@ sub main_handler {
     return DECLINED;
 }
 
-sub run_axkit_engine {
-    my ($r, $provider, $no_output) = @_;
-    
-    # get preferred stylesheet and media type
-    my ($preferred, $media) = get_style_and_media();
-    AxKit::Debug(2, "media: $media, preferred style: $preferred");
-
-    # get cache object
-    my $cache = Apache::AxKit::Cache->new($r, $r->filename() . ($AxKit::Cfg->GzipOutput ? '.gzip' : '') . ($r->path_info() || ''), $preferred, $media, $r->notes('axkit_cache_extra'));
-
-    my $recreate = 0; # regenerate from source (not cached)
-
-    my $styles = get_styles($media, $preferred, $cache, $provider);
-
-    {
-        local $^W;
-        if ($preferred && ($styles->[0]{title} ne $preferred)) {
-            # we selected a style that didn't exist.
-            # Make sure we default the cache file, otherwise
-            # we setup a potential DoS
-            AxKit::Debug(3, "resetting cache with no preferred style ($preferred ne $styles->[0]{title})");
-            $cache = Apache::AxKit::Cache->new($r, $r->filename() . ($AxKit::Cfg->GzipOutput ? '.gzip' : '') . $r->path_info(), '', $media, $r->notes('axkit_cache_extra'));
-        }
-    }
-
-    if (!$cache->exists()) {
-        AxKit::Debug(2, "cache doesn't exist");
-        # set no_cache header if cache doesn't exist due to no_cache option
-        $r->no_cache(1) if $cache->no_cache();
-        $recreate++;
-    }
-
-    if (!$recreate && $AxKit::Cfg->DependencyChecks()) {
-        $recreate = check_dependencies($r, $provider, $cache);
-    }
-
-    if (!$recreate && $r->method() eq 'POST') {
-        $recreate++;
-    }
-
-    $AxKit::Charset = $AxKit::Cfg->OutputCharset();
-
-    # Store in package variable for other modules
-    $AxKit::Cache = $cache;
-    
-    if (!$recreate) {
-        AxKit::Debug(1, "delivering cached copy - all conditions met");
-        return $cache->deliver();
-    }
-
-    AxKit::Debug(1, "some condition failed. recreating output");
-
-    # reconsecrate Apache request object (& STDOUT) into our own class
-    bless $r, 'AxKit::Apache';
-    tie *STDOUT, 'AxKit::Apache', $r;
-    
-    if (my $charset = $AxKit::Cfg->OutputCharset) {
-        AxKit::Debug(5, "Different output charset: $charset");
-        if (!$r->notes('axkit_passthru_type')) {
-            $r->content_type("text/html; charset=$charset");
-        }
-    }
-    
-    # This is here so that lookup_uri() works on the real thing
-    # that we're requesting, not on the thing plus the PATH_INFO
-    my $uri = $r->uri();
-    my $path_info = $r->path_info();
-    substr($uri, -length($path_info)) = '' if length($path_info); # trim off the end that many chars (negative offset to substr)
-    $r->uri($uri);
-    $ENV{PATH_INFO} = $path_info;
-
-    {
-        # copy styles because we blat the copy
-        my @copy = @$styles;
-        $AxKit::_CurrentStylesheets = \@copy;
-    }
-    
-    # Main grunt of the work done here...
-    my $return_code = process_request(
-        $r,
-        $provider,
-        $AxKit::_CurrentStylesheets,
-        $no_output,
-    );
-
-    save_dependencies($r, $cache);
-    
-    return $return_code;
-        
-}
-
-sub get_axkit_uri {
-    my ($uri) = @_;
-    
-    AxKit::Debug(3, "get_axkit_uri($uri)");
-    
-    my $apache = AxKit::Apache->request;
-    my $r;
-    if ($uri =~ m|^axkit:(/.*)$|) {
-        my $blurb = $1;
-        # got "axkit:/..."
-        # first check if it's actually "axkit://host[:port]/..."
-        if ($blurb =~ m|^//(?:([\w\.-]+)(:\d+)?)?(/.*)$|) {
-            my $host = $1;
-            my $port = $2;
-            my $abs_uri = $3;
-            if ($host) {
-                throw Apache::AxKit::Exception (
-                    -text => "axkit://host[:port]/ requests not supported ($uri)"
-                );
-            }
-            AxKit::Debug(4, "get_axkit_uri looking up abs (host): '$abs_uri'");
-            $r = $apache->lookup_uri(AxKit::FromUTF8($abs_uri));
-        }
-        else {
-            my $abs_uri = $blurb;
-            AxKit::Debug(4, "get_axkit_uri looking up abs: '$abs_uri'");
-            $r = $apache->lookup_uri(AxKit::FromUTF8($abs_uri));
-        }
-    }
-    elsif ($uri =~ /^axkit:(.*)$/) {
-        my $rel_uri = $1;
-        AxKit::Debug(4, "get_axkit_uri looking up '$rel_uri'");
-        $r = $apache->lookup_uri(AxKit::FromUTF8($rel_uri));
-    }
-    else {
-        throw Apache::AxKit::Exception (-text => "get_axkit_uri for non-axkit URIs is not yet supported");
-    }
-    
-    local $AxKit::Cfg = Apache::AxKit::ConfigReader->new($r);
-    local $AxKit::Cache;
-
-    my $provider = Apache::AxKit::Provider->new_content_provider($r);
-    
-    my $result_code = eval {
-        run_axkit_engine(
-            $r,
-            $provider,
-            1, # no output
-        );
-    };
-    if ($@) {
-        my $E = $@;
-        if ($E->isa('Apache::AxKit::Exception::Declined')) {
-            if ($AxKit::Cfg && $AxKit::Cfg->LogDeclines()) {
-                $r->log->warn("[AxKit] [DECLINED] $E->{reason}")
-                        if $E->{reason};
-            }
-            AxKit::Debug(4, "[DECLINED] From: $E->{-file} : $E->{-line}");
-            
-            my $str = '';
-            eval {
-                my $fh = $provider->get_fh;
-                local $/;
-                $str = <$fh>;
-                $r->pnotes('xml_string', $str);
-                $result_code = OK;
-            };
-            if ($@) {
-                eval {
-                    my $pstr = $provider->get_strref;
-                    $str = $$pstr;
-                    $r->pnotes('xml_string', $str);
-                    $result_code = OK;
-                };
-                if ($@) {
-                    $result_code = 'provider_declined';
-                }
-            }
-        }
-        else {
-            # Some other type of exception
-            $E->throw;
-        }
-    }
-    
-    if ($result_code == OK) {
-        # results now in $r->pnotes('xml_string') - probably...
-        if ( my $str = $r->pnotes('xml_string') ) {
-            # NB: this is naive, but "0" isn't valid XML anyway
-            $r->pnotes('xml_string', undef);
-            AxKit::Debug(10, "get_axkit_uri returning (str): '$str'");
-            return $str;
-        }
-        elsif ($r->pnotes('dom_tree')) {
-            my $str = $r->pnotes('dom_tree')->toString;
-            $r->pnotes('dom_tree', undef);
-            AxKit::Debug(10, "get_axkit_uri returning (dom): '$str'");
-            return $str;
-        }
-        else {
-            throw Apache::AxKit::Exception (
-                -text => "$uri internal request didn't store anything in dom_tree or xml_string",
-                );
-        }
-    }
-    elsif ($result_code == DECLINED) {
-        # probably came from the cache system. Try and read it.
-        AxKit::Debug(10, "get_axkit_uri cache read");
-        return $AxKit::Cache->read();
-    }
-    else {
-        throw Apache::AxKit::Exception (
-            -text => "$uri internal request returned unknown result code: ".$result_code,
-        );
-    }
-}
-
 sub process_error {
     my ($r, $E, $error_styles) = @_;
     
@@ -598,8 +402,24 @@ sub process_error {
     $AxKit::Cache = Apache::AxKit::Cache->new($r, 'error', '', '', '');
     
     $r->content_type("text/html; charset=UTF-8"); # set a default for errors
+    
+    my $error = '<error><file>' .
+            xml_escape($r->filename) . '</file><msg>' .
+            xml_escape($E->{-text}) . '</msg>' .
+            '<stack_trace><bt level="0">'.
+            '<file>' . xml_escape($E->{'-file'}) . '</file>' .
+            '<line>' . xml_escape($E->{'-line'}) . '</line>' .
+            '</bt>';
+    
+    my $i = 1;
+    for my $stack (@{$E->stacktrace_list}) {
+        $error .= '<bt level="' . $i++ . '">' .
+                '<file>' . xml_escape($stack->{'-file'}) . '</file>' .
+                '<line>' . xml_escape($stack->{'-line'}) . '</line>' .
+                '</bt>';
+    }
 
-    my $error = $E->as_xml($r->filename);
+    $error .= '</stack_trace></error>';
 
     my $provider = Apache::AxKit::Provider::Scalar->new(
             $r, $error, $error_styles
@@ -662,53 +482,18 @@ sub reset_stylesheets {
 }
 
 sub process_request {
-    my ($r, $provider, $styles, $no_output) = @_;
+    my ($r, $provider, $styles) = @_;
     my $result_code = OK;
-
+    
     my $num_styles = 0;
     for my $style (@$styles) {
         AxKit::Debug(4, "styles: ", $style->{module}, "(", $style->{href}, ")");
         $num_styles++;
     }
 
-    my $interm_prefix;
-    my $interm_count = 0;
-    if ($AxKit::Cfg->TraceIntermediate) {
-        my $id = $r->notes('AxRequestID');
-        $interm_prefix = ($id?$id:$r->uri);
-        $interm_prefix =~ s{%}{%25}g;
-        $interm_prefix =~ s{/}{%2f}g;
-        my $ti = $AxKit::Cfg->TraceIntermediate;
-        if (defined $id) {
-                $interm_prefix = substr($interm_prefix,-1).'/'.substr($interm_prefix,-3,2).'/'.substr($interm_prefix,0,-3).'/';
-                mkdir($ti.'/'.substr($interm_prefix,0,1),0777);
-                mkdir($ti.'/'.substr($interm_prefix,0,4),0777);
-                mkdir($ti.'/'.$interm_prefix,0777);
-        } else {
-                $interm_prefix =~ s/[^0-9a-zA-Z.,_|-]/_/g;
-                $interm_prefix .= '.';
-        }
-        $interm_prefix = $ti.'/'.$interm_prefix;
-    
-        if ($interm_prefix) {
-            my $fh = Apache->gensym();
-            if (sysopen($fh, $interm_prefix.$interm_count, O_WRONLY|O_CREAT|O_TRUNC)) {
-                syswrite($fh,${$provider->get_strref});
-                close($fh);
-                $interm_count++;
-            } else {
-                AxKit::Debug(1,"could not open $interm_prefix$interm_count for writing: $!");
-            }
-        }
-    }
-
     while (@$styles) {
         my $style = shift @$styles;
-        
-        my $num_left = @$styles;
-        my $output_to_browser = ($num_left == 0 && !$no_output);
-
-        my $styleprovider = Apache::AxKit::Provider->new_style_provider(
+        my $styleprovider = Apache::AxKit::Provider->new(
                 $r,
                 uri => $style->{href},
                 );
@@ -719,15 +504,9 @@ sub process_request {
 
         my $mapto = $style->{module};
 
-        # if no module is give AxKit should use the default modules
-        # from the server config.
-        unless ( $mapto ) {
-            my $mapping = $AxKit::Cfg->StyleMap;
-            $mapto = $mapping->{$style->{type}};
-        }
-
-        AxKit::Debug(3, "about to execute: $mapto\::handler");
         AxKit::load_module($mapto);
+        
+        AxKit::Debug(3, "about to execute: $mapto\::handler");
 
         my $method = "handler";
         if (defined &{"$mapto\::$method"}) {
@@ -737,13 +516,12 @@ sub process_request {
                         );
             }
             my $retval = $mapto->$method(
-                    $r,
-                    $provider,
-                    $styleprovider,
-                    $output_to_browser,
+                    $r, 
+                    $provider, 
+                    $styleprovider, 
+                    !@$styles, # any more left?
                     );
-	    AxKit::Debug(5, "$mapto\::handler finished with code $retval");
-            $result_code = $retval if $retval != OK;
+	        $result_code = $retval if $retval != OK;
         }
         else {
             throw Apache::AxKit::Exception::Error(
@@ -751,44 +529,6 @@ sub process_request {
                 );
         }
 
-        if ($interm_prefix) {
-            my $fh = Apache->gensym();
-            if (open($fh, '>'.$interm_prefix.$interm_count)) {
-                my $xmlstr;
-                if ($AxKit::Cfg->DebugTidy) {
-                    eval {
-                        require XML::LibXML;
-                        my $parser = new XML::LibXML();
-                        $parser->keep_blanks(0);
-                        if (my $dom_tree = $r->pnotes('dom_tree')) {
-                            $xmlstr = $dom_tree->toString;
-                        } elsif ($r->pnotes('xml_string')) {
-                            $xmlstr = $r->pnotes('xml_string');
-                        } else {
-                            die;
-                        }
-                        local $XML::LibXML::setTagCompression = 1;
-                        $xmlstr = $parser->parse_string($xmlstr,$r->uri)->toString(1);
-                    };
-                    AxKit::Debug(1,"AxDebugTidy unavailable for XML: $@") if $@;
-                }
-                if (!defined $xmlstr) {
-                    if (my $dom_tree = $r->pnotes('dom_tree')) {
-                        $xmlstr = $dom_tree->toString;
-                    } elsif ($r->pnotes('xml_string')) {
-                        $xmlstr = $r->pnotes('xml_string');
-                    } else {
-                        $xmlstr = "<?xml version='1.0'?>\n<empty reason='no data found'/>\n";
-                    }
-                }
-                print($fh $xmlstr);
-                close($fh);
-                $interm_count++;
-            } else {
-                AxKit::Debug(1,"could not open $interm_prefix$interm_count for writing: $!");
-            }
-        }
-
         AxKit::Debug(3, "execution of: $mapto\::$method finished");
 
         last if $r->notes('axkit_passthru');
@@ -829,12 +569,12 @@ sub get_styles {
     else {
         AxKit::Debug(3, "styles not cached - calling \$provider->get_styles()");
         my $styles = $provider->get_styles($media, $style);
-
+        
         $AxKit::Stash{$key} = {
             styles => $styles,
             mtime => $provider->mtime(),
             };
-
+        
         return $styles;
     }
 }
@@ -870,9 +610,9 @@ sub check_dependencies {
 
 sub save_dependencies {
     my ($r, $cache) = @_;
-
+    
     return if $cache->no_cache();
-
+    
     eval {
         my @depends = get_depends();
         my $depend_cache = Apache::AxKit::Cache->new($r, $cache->key(), '.depends');
@@ -886,21 +626,11 @@ sub save_dependencies {
 sub deliver_to_browser {
     my ($r, $result_code) = @_;
     $result_code ||= OK;
-    
-    AxKit::Debug(4, "delivering to browser");
 
-    if (length($r->pnotes('xml_string'))) {
-        # ok, data is in xml_string
-        AxKit::Debug(4, "Delivering xml_string");
-    }
-    elsif ($r->pnotes('dom_tree')) {
-        AxKit::Debug(4, "Delivering dom_tree");
+    if (not $r->pnotes('xml_string') and $r->pnotes('dom_tree')) {
         $r->pnotes('xml_string', $r->pnotes('dom_tree')->toString );
     }
-    else {
-        AxKit::Debug(5, "Possible cache delivery coming up...");
-    }
-
+    
     if ($r->content_type eq 'changeme' && !$r->notes('axkit_passthru_type')) {
         $AxKit::Cfg->AllowOutputCharset(1);
         $r->content_type('text/html; charset=' . ($AxKit::Cfg->OutputCharset || "UTF-8"));
@@ -908,43 +638,42 @@ sub deliver_to_browser {
     elsif ($r->notes('axkit_passthru_type')) {
         $r->content_type($AxKit::OrigType);
     }
-
+    
     if (my $charset = $AxKit::Cfg->OutputCharset()) {
         my $ct = $r->content_type;
         $ct =~ s/charset=.*?(;|$)/charset=$charset/i;
         $r->content_type($ct);
     }
-
-    if ($result_code != OK && $result_code != 200) {
+            
+    if ($result_code != OK) {
     	# no caching - probably makes no sense, and will be turned off
-        # anyways, as currently only XSP pages allow to send custom responses
+	    # anyways, as currently only XSP pages allow to send custom responses
     	AxKit::Debug(4,"sending custom response: $result_code");
         my ($transformer, $doit) = get_output_transformer();
         if ($doit) {
-            $r->custom_response($result_code,$transformer->($r->pnotes('xml_string') || ''));
+        	$r->custom_response($result_code,$transformer->($r->pnotes('xml_string')));
         }
-        else {
-            $r->custom_response($result_code,$r->pnotes('xml_string') || '');
-        }
-        return $result_code;
+         else {
+         	$r->custom_response($result_code,$r->pnotes('xml_string'));
+	    }
+	    return $result_code;
     }
 
     if ($AxKit::Cache->no_cache() ||
-        lc($r->dir_config('Filter')) eq 'on' ||
-        $r->method() eq 'POST')
-    {
+            lc($r->dir_config('Filter')) eq 'on' ||
+            $r->method() eq 'POST') {
         AxKit::Debug(4, "writing xml string to browser");
         my ($transformer, $doit) = get_output_transformer();
         if ($AxKit::Cfg->DoGzip) {
             AxKit::Debug(4, 'Sending gzipped xml string to browser');
             AxKit::Apache::send_http_header($r);
             if ($doit) {
-                $r->print( unpack($]>5.00555?"U0A*":"A*", Compress::Zlib::memGzip( 
+                $r->print( Compress::Zlib::memGzip(
                          $transformer->( $r->pnotes('xml_string') )
-                         ) ) );
+                         ) );
             }
             else {
-                $r->print( unpack($]>5.00555?"U0A*":"A*", Compress::Zlib::memGzip( $r->pnotes('xml_string') ) ) );
+                $r->print( Compress::Zlib::memGzip( $r->pnotes('xml_string') ) );
             }
         }
         else {
@@ -1054,7 +783,7 @@ sub xml_escape {
 #########################################################################
 
 package AxKit::Apache;
-use vars qw/@ISA $PNOTES/;
+use vars qw/@ISA/;
 use Apache;
 use Fcntl qw(:DEFAULT);
 @ISA = ('Apache');
@@ -1088,7 +817,7 @@ sub print {
         $self->pnotes('xml_string', '');
         $self->notes('resetstring', 0);
     }
-
+    
     my $current = $self->pnotes('xml_string');
     $self->pnotes('xml_string', $current . join('', @_));
 }
@@ -1122,18 +851,6 @@ sub send_http_header {
     $self->SUPER::send_http_header;
 }
 
-sub pnotes {
-    my $self = shift;
-    if (!@_) {
-        return $PNOTES;
-    }
-    my $key = shift;
-    if (@_) {
-        return $PNOTES->{$key} = $_[0];
-    }
-    return $PNOTES->{$key};
-}
-
 1;
 
 package AxKit::ApacheDebug;
@@ -1237,24 +954,6 @@ use this option on a live server.
 
     AxDebugLevel 5
 
-=head2 AxTraceIntermediate
-
-With this option you advise AxKit to store the result of each transformation
-request in a special directory for debugging. This directory must exist and must
-be writeable by the httpd. The files are stored with their full uri, replacing
-slashes with '|', and appending a number indicating the transformation step.
-'.0' is the xml after the first transformation.
-
-    AxTraceIntermediate /tmp/axkit-trace
-
-=head2 AxDebugTidy
-
-With this option you advise AxKit to tidy up debug dumps like XSP scripts or the
-XML files generated by AxTraceIntermediate. Be aware that this can slow down requests
-considerably, but often it is much easier to spot errors with this enabled.
-
-    AxDebugTidy On
-
 =head2 AxStackTrace
 
 This FLAG option says whether to maintain a stack trace with every exception.
@@ -1329,29 +1028,6 @@ only enable if you also enable AxTranslateOutput.
 
     AxOutputCharset iso-8859-1
 
-=head2 AxExternalEncoding
-
-This directive specifies the character encoding used outside of AxKit.
-Internally, AxKit strictly uses UTF-8 (remember that when you write Taglibs!),
-but file names on the file system and URIs requested by browsers may use
-a different encoding, e.g. ISO-8859-15 for most of Europe.
-
-This is a server-global directive, so only use it within <VirtualHost ...>
-containers or on the root level. As a side effect, this option allows you to
-work with non-ASCII chars in URLs even outside of AxKit. Some Browsers may
-send URLs in their local charset although the link was encoded in UTF-8, and
-others always send UTF-8 encoded URLs, regardless of link encoding. If you set
-this option, AxKit will intercept each request and check if the URL came
-encoded in UTF-8. If so, AxKit transforms it to the character encoding
-specified here before Apache gets to resolve the request, so Apache will
-find the file, even if it is a non-AxKit file.
-
-Note: This does not effect the _contents_ of documents, they have their own
-encoding specifier (in the <?xml ...?> line). For now, it only affects file
-names and URL processing.
-
-    AxExternalEncoding ISO-8859-1
-
 =head2 AxAddOutputTransformer
 
 Output transformers are applied just before output is sent to the browser.
@@ -1381,7 +1057,7 @@ down.
    # This directive takes no arguments
    AxResetOutputTransformers
 
-=head2 AxErrorStylesheet
+ =head2 AxErrorStylesheet
 
 If an error occurs during processing that throws an exception, the
 exception handler will try and find an ErrorStylesheet to use to process
@@ -1422,25 +1098,6 @@ If you prefix the module name with a + sign, it will be pre-loaded on
 server startup (assuming that the config directive is in a httpd.conf,
 rather than a .htaccess file).
 
-=head2 AxIgnoreStylePI
-
-Turn off parsing and overriding stylesheet selection for XML files containing
-an C<xml-stylesheet> processing instruction at the start of the file. This is
-a FLAG option - On or Off. The default value is "Off".
-
-  AxIgnoreStylePI On
-
-=head2 AxHandleDirs
-
-Enable this option to allow AxKit to process directories. Creates an XML document
-with the contents of the requested directory. Look at sample output to
-discover that the format is straightforward and easy to understand.
-
-  AxHandleDirs On
-
-A DTD for the output of AxHandleDirs is at 
-L<http://axkit.org/dtd/axhandledirs.dtd>
-
 =head2 AxStyle
 
 A default stylesheet title to use. This is useful when a single XML
@@ -1488,10 +1145,8 @@ a directory processed by a different XSLT engine.
 
 There are several directives specifically designed to allow you to build
 a flexible sitemap that specifies how XML files get processed on your
-system.
-
-B<Note:> <?xml-stylesheet?> directives in your XML files override these
-directives unless you enable the AxIgnoreStylePI option listed above.
+system. These directives are used only if your XML file does not have the
+<?xml-stylesheet?> directives.
 
 =head2 AxAddProcessor
 
@@ -1613,18 +1268,15 @@ These directives all take as a single argument, the name of a module
 to load in place of the default. They are:
 
     AxConfigReader
-    AxContentProvider
-    AxStyleProvider
+    AxProvider
     AxCacheModule
 
 The ConfigReader module returns information about various configuration
 options. Currently it takes most of its information from the above
 mentioned configuration directives, or from PerlSetVar.
 
-The Provider modules are the means by which AxKit gets its resources from.
-ContentProviders deliver up the document to be processed, while StyleProviders
-are used to get the data for any stylesheets that will be applied.
-The default Provider for each simply picks up files from the filesystem, but
+The Provider module is the means by which AxKit gets its resources from.
+The default Provider simply picks up files from the filesystem, but
 alternate providers could pull the information from a DBMS, or perhaps
 create some XML structure for directories. There currently exists one
 alternate Provider module, which allows AxKit to work as a recipient
